package minica;

import java.io.File;
import java.io.IOException;
import java.util.Locale;

import minica.crypto.CryptoInformationProvider;
import minica.crypto.KeyStoreProvider;

import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.jface.dialogs.DialogSettings;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This class controls all aspects of the application's execution
 */
public class Application implements IApplication {

	private static String			homedir;

	private static IDialogSettings	settings;

	// TODO add save and load options to file menu
	// TODO remove ... dots in menubar
	// TODO add about dialog, include link to fam fam icons as required by it's license!
	// TODO move "search batch file" button in the same line as fielnmae and text field
	// TODO set batch mode page complete when filename is entered via button

	public static String getHome() {
		return homedir;
	}

	public static IDialogSettings getSettings() {
		return settings;
	}

	public Application() {
		// Construct the homedir, using proper file separator values for the current system.
		homedir = System.getProperty("user.home") + System.getProperty("file.separator") + "minica"
				+ System.getProperty("file.separator");
		File directory = new File(homedir);
		if (!directory.exists()) {
			directory.mkdir();
		}
		String settingsFileName = getHome() + "settings.xml";
		File settingsFile = new File(settingsFileName);
		
		try {

			//Read Config from File

			if (settingsFile.exists() && settingsFile.canRead()) {

				settings = new DialogSettings("root");
				settings.load(settingsFileName);

			} else {
				//New Config
				settings = new DialogSettings("root");
				String[] algos = new String[] { "RSA,FlexiCore,false", "DSA,FlexiCore,false", "EC,FlexiEC,true" };
				settings.put("WorkingAlgorithms", algos);

				String[] signatures = new String[] { "MD5withRSA,FlexiCore", "RIPEMD160withRSA,FlexiCore", 
						"SHA1withRSA,FlexiCore", "SHA224withRSA,FlexiCore",
						"SHA256withRSA,FlexiCore", "SHA384withRSA,FlexiCore", "SHA512withRSA,FlexiCore",
						"SHA1withDSA,FlexiCore", "SHA1withECDSA,FlexiEC", "SHA224withECDSA,FlexiEC",
						"SHA256withECDSA,FlexiEC", "SHA384withECDSA,FlexiEC", "SHA512withECDSA,FlexiEC",
						"SHA1withECDSA,FlexiEC" };
				settings.put("WorkingSignatures", signatures);
				
				//settings.put("KeyStorePassword", "password");
				String[] oneClickDefaultDN = new String[] {"CommonName","MyOrgUnit","MyOrg",Locale.getDefault().getCountry()};
				settings.put("OneClickDefaultDN",oneClickDefaultDN);
				settings.put("OneClickDefaultKeyAlgorithm", "RSA");
				settings.put("OneClickDefaultSignatureAlgorithm", "SHA1withRSA");
				settings.save(settingsFileName);
			}

		} catch (IOException e) {
			e.printStackTrace();
			// TODO Auto-generated catch block
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.equinox.app.IApplication#start(org.eclipse.equinox.app.IApplicationContext)
	 */
	public Object start(IApplicationContext context) throws Exception {
		Display display = PlatformUI.createDisplay();
		try {
			// initialize the providers
			CryptoInformationProvider.getInstance();
			int returnCode = PlatformUI.createAndRunWorkbench(display, new ApplicationWorkbenchAdvisor());
			this.cleanup();
			if (returnCode == PlatformUI.RETURN_RESTART)
				return IApplication.EXIT_RESTART;
			else
				return IApplication.EXIT_OK;
		} finally {
			display.dispose();
		}

	}

	/* (non-Javadoc)
	 * @see org.eclipse.equinox.app.IApplication#stop()
	 */
	public void stop() {
		this.cleanup();
		final IWorkbench workbench = PlatformUI.getWorkbench();
		if (workbench == null)
			return;
		final Display display = workbench.getDisplay();
		display.syncExec(new Runnable() {
			public void run() {
				if (!display.isDisposed())
					workbench.close();
			}
		});
	}

	private void cleanup() {
		// save the cip information
		CryptoInformationProvider cip = CryptoInformationProvider.getInstance();
		//System.out.println("Application.cleanup() - calling cip.saveDb()");
		try {
			cip.saveDb();
		} catch (Exception e) {
			e.printStackTrace();
		}
		// save the datastructure
		KeyStoreProvider ksp = KeyStoreProvider.getInstance();
		//System.out.println("Application.cleanup() - calling ksp.serialize()");
		try {
			ksp.serialize();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
