package minica.cert;

import java.math.BigInteger;
import java.util.Arrays;
import java.util.GregorianCalendar;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This Class encapsulates all the data needed to create a Certificate using
 * a CreateCertificateJob.
 * @author stephan
 *
 */
public class CertificateJobDescription {

	public static final int	RootCertificate		= 0;

	public static final int	CACertificate		= 1;

	public static final int	ClientCertificate	= 2;

	public static final int	BatchMode			= 3;

	public static final int	OneClick			= 4;

	public static int getCACertificate() {
		return CACertificate;
	}

	public static int getClientCertificate() {
		return ClientCertificate;
	}

	public static int getRootCertificate() {
		return RootCertificate;
	}

	private boolean				complete			= false;

	private int					certificateType;

	private String				issuerPassword;

	private String				pkcs12Password;

	private String				fileName;

	private boolean[]			keyUsage;

	private String				algorithm;

	private int					keyLength;

	private String				commonName;

	private String				organisation;

	private String				organisationalUnit;

	private String				country;

	private GregorianCalendar	validNotBefore;

	private GregorianCalendar	validNotAfter;

	private boolean				crl					= false;

	private boolean				exportAlternates	= false;

	private int					pathlength;

	private String				crlDistributionPoint;

	private String				signatureAlgorithm;

	private BigInteger			serial;

	private String				issuer;

	private String				ecCurve;

	//private BigInteger serial = new BigInteger(serial2);

	/**
	 * Clone this job descrition.
	 * @param def
	 */
	public CertificateJobDescription(CertificateJobDescription def) {
		this.algorithm = def.getAlgorithm();
		this.certificateType = def.getCertificateType();
		this.commonName = def.getCommonName();
		this.complete = def.isComplete();
		this.country = def.getCountry();
		this.crl = def.isCrl();
		this.crlDistributionPoint = def.getCrlDistributionPoint();
		this.ecCurve = def.getECCurve();
		this.fileName = def.getFileName();
		this.issuer = def.getIssuer();
		this.issuerPassword = def.getIssuerPassword();
		this.keyLength = def.getKeyLength();
		this.keyUsage = def.getKeyUsage();
		this.organisation = def.getOrganisation();
		this.organisationalUnit = def.getOrganisationalUnit();
		this.pathlength = def.getPathlength();
		this.pkcs12Password = def.getPkcs12Password();
		this.serial = def.getSerial();
		this.signatureAlgorithm = def.getSignatureAlgorithm();
		this.validNotAfter = def.getValidNotAfter();
		this.validNotBefore = def.getValidNotBefore();
		this.complete = true;
	}

	public CertificateJobDescription(int type) {
		certificateType = type;
	}

	public CertificateJobDescription(String f_name) {
		fileName = f_name;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CertificateJobDescription other = (CertificateJobDescription) obj;
		if (algorithm == null) {
			if (other.algorithm != null)
				return false;
		} else if (!algorithm.equals(other.algorithm))
			return false;
		if (certificateType != other.certificateType)
			return false;
		if (commonName == null) {
			if (other.commonName != null)
				return false;
		} else if (!commonName.equals(other.commonName))
			return false;
		if (complete != other.complete)
			return false;
		if (country == null) {
			if (other.country != null)
				return false;
		} else if (!country.equals(other.country))
			return false;
		if (crl != other.crl)
			return false;
		if (crlDistributionPoint == null) {
			if (other.crlDistributionPoint != null)
				return false;
		} else if (!crlDistributionPoint.equals(other.crlDistributionPoint))
			return false;
		if (ecCurve == null) {
			if (other.ecCurve != null)
				return false;
		} else if (!ecCurve.equals(other.ecCurve))
			return false;
		if (fileName == null) {
			if (other.fileName != null)
				return false;
		} else if (!fileName.equals(other.fileName))
			return false;
		if (issuer == null) {
			if (other.issuer != null)
				return false;
		} else if (!issuer.equals(other.issuer))
			return false;
		if (issuerPassword == null) {
			if (other.issuerPassword != null)
				return false;
		} else if (!issuerPassword.equals(other.issuerPassword))
			return false;
		if (keyLength != other.keyLength)
			return false;
		if (!Arrays.equals(keyUsage, other.keyUsage))
			return false;
		if (organisation == null) {
			if (other.organisation != null)
				return false;
		} else if (!organisation.equals(other.organisation))
			return false;
		if (organisationalUnit == null) {
			if (other.organisationalUnit != null)
				return false;
		} else if (!organisationalUnit.equals(other.organisationalUnit))
			return false;
		if (pathlength != other.pathlength)
			return false;
		if (pkcs12Password == null) {
			if (other.pkcs12Password != null)
				return false;
		} else if (!pkcs12Password.equals(other.pkcs12Password))
			return false;
		if (serial == null) {
			if (other.serial != null)
				return false;
		} else if (!serial.equals(other.serial))
			return false;
		if (signatureAlgorithm == null) {
			if (other.signatureAlgorithm != null)
				return false;
		} else if (!signatureAlgorithm.equals(other.signatureAlgorithm))
			return false;
		if (validNotAfter == null) {
			if (other.validNotAfter != null)
				return false;
		} else if (!validNotAfter.equals(other.validNotAfter))
			return false;
		if (validNotBefore == null) {
			if (other.validNotBefore != null)
				return false;
		} else if (!validNotBefore.equals(other.validNotBefore))
			return false;
		return true;
	}

	public String getAlgorithm() {
		return algorithm;
	}

	public int getCertificateType() {
		return certificateType;
	}

	public String getCommonName() {
		return commonName;
	}

	public String getCountry() {
		return country;
	}

	public String getCrlDistributionPoint() {
		return crlDistributionPoint;
	}

	public String getECCurve() {
		return ecCurve;
	}

	public String getFileName() {
		return fileName;
	}

	public String getIssuer() {
		return issuer;
	}

	public String getIssuerPassword() {
		return issuerPassword;
	}

	public int getKeyLength() {
		return keyLength;
	}

	public boolean[] getKeyUsage() {
		return keyUsage;
	}

	public String getOrganisation() {
		return organisation;
	}

	public String getOrganisationalUnit() {
		return organisationalUnit;
	}

	public int getPathlength() {
		return pathlength;
	}

	public String getPkcs12Password() {
		return pkcs12Password;
	}

	public BigInteger getSerial() {
		return serial;
	}

	public String getSignatureAlgorithm() {
		return signatureAlgorithm;
	}

	public int getType() {
		return certificateType;
	}

	public GregorianCalendar getValidNotAfter() {
		return validNotAfter;
	}

	public GregorianCalendar getValidNotBefore() {
		return validNotBefore;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((algorithm == null) ? 0 : algorithm.hashCode());
		result = prime * result + certificateType;
		result = prime * result + ((commonName == null) ? 0 : commonName.hashCode());
		result = prime * result + (complete ? 1231 : 1237);
		result = prime * result + ((country == null) ? 0 : country.hashCode());
		result = prime * result + (crl ? 1231 : 1237);
		result = prime * result + ((crlDistributionPoint == null) ? 0 : crlDistributionPoint.hashCode());
		result = prime * result + ((ecCurve == null) ? 0 : ecCurve.hashCode());
		result = prime * result + ((fileName == null) ? 0 : fileName.hashCode());
		result = prime * result + ((issuer == null) ? 0 : issuer.hashCode());
		result = prime * result + ((issuerPassword == null) ? 0 : issuerPassword.hashCode());
		result = prime * result + keyLength;
		result = prime * result + Arrays.hashCode(keyUsage);
		result = prime * result + ((organisation == null) ? 0 : organisation.hashCode());
		result = prime * result + ((organisationalUnit == null) ? 0 : organisationalUnit.hashCode());
		result = prime * result + pathlength;
		result = prime * result + ((pkcs12Password == null) ? 0 : pkcs12Password.hashCode());
		result = prime * result + ((serial == null) ? 0 : serial.hashCode());
		result = prime * result + ((signatureAlgorithm == null) ? 0 : signatureAlgorithm.hashCode());
		result = prime * result + ((validNotAfter == null) ? 0 : validNotAfter.hashCode());
		result = prime * result + ((validNotBefore == null) ? 0 : validNotBefore.hashCode());
		return result;
	}

	public boolean isComplete() {
		return complete;
	}

	public boolean isCrl() {
		return crl;
	}

	/**
	 * @param algorithm the algorithm to set
	 */
	public void setAlgorithm(String algorithm) {
		this.algorithm = algorithm;
	}

	public void setCiperData(String algo, int length) {
		algorithm = algo;
		keyLength = length;
	}

	/**
	 * @param commonName the commonName to set
	 */
	public void setCommonName(String commonName) {
		this.commonName = commonName;
	}

	public void setComplete() {
		complete = true;
	}

	/**
	 * @param country the country to set
	 */
	public void setCountry(String country) {
		this.country = country;
	}

	public void setCRL(boolean value, int length, String crlDistribPoint) {
		crl = value;
		pathlength = length;
		crlDistributionPoint = crlDistribPoint;
	}

	public void setECCurve(String text) {
		ecCurve = text;

	}

	public void setFileName(String f_name) {
		fileName = f_name;
	}

	public void setInComplete() {
		complete = false;

	}

	public void setIssuer(String issuer) {
		this.issuer = issuer;
	}

	public void setIssuerPassword(String pw) {
		issuerPassword = pw;
	}

	/**
	 * @param keyLength the keyLength to set
	 */
	public void setKeyLength(int keyLength) {
		this.keyLength = keyLength;
	}

	public void setKeyUsage(boolean[] usage) {
		keyUsage = usage;
	}

	/**
	 * @param organisation the organisation to set
	 */
	public void setOrganisation(String organisation) {
		this.organisation = organisation;
	}

	/**
	 * @param organisationalUnit the organisationalUnit to set
	 */
	public void setOrganisationalUnit(String organisationalUnit) {
		this.organisationalUnit = organisationalUnit;
	}

	public void setPathLength(int length) {
		this.pathlength = length;
	}

	public void setPKCS12Password(String password) {
		pkcs12Password = password;
	}

	public void setSerial(BigInteger serial) {
		this.serial = serial;
	}

	public void setSignatureAlgorithm(String algo) {
		signatureAlgorithm = algo;
	}

	public void setSubject(String[] subject) {
		commonName = subject[0];
		organisationalUnit = subject[1];
		organisation = subject[2];
		country = subject[3];
	}

	public void setType(int type) {
		this.certificateType = type;
	}

	public void setValidityPeriod(GregorianCalendar[] validity) {
		validNotBefore = validity[0];
		validNotAfter = validity[1];
	}

	/**
	 * @param validNotAfter the validNotAfter to set
	 */
	public void setValidNotAfter(GregorianCalendar validNotAfter) {
		this.validNotAfter = validNotAfter;
	}

	/**
	 * @param validNotBefore the validNotBefore to set
	 */
	public void setValidNotBefore(GregorianCalendar validNotBefore) {
		this.validNotBefore = validNotBefore;
	}

	public String toString() {
		String br = System.getProperty("line.separator");
		return ("Filename: " + fileName + br + "KeyAlgorithm: " + algorithm + br + "Keylength: " + keyLength + br
				+ "Signature Algorithm: " + signatureAlgorithm + br + "PKCS12PW: " + pkcs12Password + br + "CN: "
				+ commonName + br + "OU: " + organisationalUnit + br + "O: " + organisation + br + "C: " + country + br
				+ "Valid Not before: " + validNotBefore.getTime() + br + "Vald not after: " + validNotAfter.getTime()
				+ br + "Pathlength: " + pathlength + br + "CRL: " + crl + br + "CRL Distribution Point: " + crlDistributionPoint);
	}

	/**
	 * @return the exportAlternates
	 */
	public boolean isExportAlternates() {
		return exportAlternates;
	}

	/**
	 * @param exportAlternates the exportAlternates to set
	 */
	public void setExportAlternates(boolean exportAlternates) {
		this.exportAlternates = exportAlternates;
	}

}
