package minica.crypto;

import java.security.KeyPairGenerator;
import java.security.Provider;
import java.security.Security;
import java.security.Provider.Service;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * Detect available asymmetric algorithms.
 * @author tex
 *
 */
public class AsymDetectionJob extends Job {

	public AsymDetectionJob(String name) {
		super(name);
	}

	/**
	 * Test is a given keysize, provider, algorithm combination is working.
	 * @param algorithm
	 * @param provider
	 * @param keysize
	 * @return true if the algorithm works with this keysize
	 */
	private boolean testKeySize(String algorithm, Provider provider, int keysize) {
		KeyPairGenerator kpg;
		try {
			kpg = KeyPairGenerator.getInstance(algorithm, provider);
			kpg.initialize(keysize);
		} catch (Exception e) {
			// something went wrong
			return false;
		}
		return true;
	}

	@Override
	protected IStatus run(IProgressMonitor arg0) {
		CryptoInformationProvider cip = CryptoInformationProvider.getInstance();
		Provider[] providers = Security.getProviders();
		for (int i = 0; i < providers.length; i++) {
			Set<Service> services = providers[i].getServices();
			for (Service s : services) {
				if (s.getType().equals("KeyPairGenerator")) {
					int max = 2049;
					// eliptic curve key generation with too big
					// keys takes forever.
					if (s.getProvider().getName().contains("EC")) {
						max = 600;
					}
					for (int j = 128; j < max; j += 128) {
						if (this.testKeySize(s.getAlgorithm(), providers[i], j)) {
							cip.addValidAsymmKeySize(s.getAlgorithm(), j);
						}
					}
				}
			}
		}
		//System.out.println("DEBUG: AsymDetectionJob finished");
		return Status.OK_STATUS;
	}

}
