package minica.crypto;

import javax.swing.event.ChangeEvent;

import codec.x509.X509Certificate;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * A Client Certificate.
 * @author tex
 *
 */
public class ClientCertificate extends AbstractCertificate implements IClientCertificate {

	public ClientCertificate(X509Certificate cert, ICaCertificate parent) {
		this.cert = cert;
		this.parentCert = parent;
		/*
		 * Important: If we have a parent we'll add ourselves to
		 * it's list of certificates. The parent in turn will
		 * register itself in our list of listeners so we can tell him
		 * if anything happens to us (new sub-certs, etc.).
		 * Nice, 'eh? ;D 
		 */
		if (this.parentCert != null) {
			this.parentCert.addCert(this);
		}
		if (this.cert != null) {
			this.pkcs12Filename = this.cert.getSerialNumber().toString();
		}
		this.stateChanged(new ChangeEvent(this));
	}

	public boolean contains(Object o) {
		if (this.equals(o)) {
			return true;
		} else {
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final ClientCertificate other = (ClientCertificate) obj;
		if (cert == null) {
			if (other.cert != null)
				return false;
		} else if (!cert.equals(other.cert))
			return false;
		return true;
	}

	public IClientCertificate get(Object o) {
		if (this.equals(o)) {
			return this;
		} else {
			return null;
		}
	}

	public X509Certificate getCertificate() {
		return this.cert;
	}

	public X509Certificate getClientCertificate() {
		return this.cert;
	}

	public ICertificate getParent() {
		return this.getParentCertificate();
	}

	public ICaCertificate getParentCertificate() {
		return this.parentCert;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cert == null) ? 0 : cert.hashCode());
		return result;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("ClientCertificate [");
		sb.append("X509Certificate: ");
		sb.append(this.cert.toString());
		sb.append("]");
		return sb.toString();
	}

}
