package minica.crypto;

import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateEncodingException;

import minica.Application;
import codec.x509.X509Certificate;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This is the Class responsible for Digest Generation and all Signing Issues
 * @author stephan
 *
 */
public class FlexiDigestAndSignatureGenerator {

	private MessageDigest	digest;

	private Signature		sig;

	public FlexiDigestAndSignatureGenerator(String digestalgo, String sigalgo) throws NoSuchAlgorithmException,
			NoSuchProviderException {
		digest = MessageDigest.getInstance(digestalgo);

		String[] algos = Application.getSettings().getArray("WorkingSignatures");
		for (int i = 0; i < algos.length; i++) {
			String current = algos[i];
			int cut = current.indexOf(",");
			String algo = current.substring(0, cut);
			String provider = current.substring(cut + 1, current.length());
			//System.out.println("Digest/Sig Generator: " + algo + " " + provider);
			if (sigalgo.equals(algo)) {

				sig = Signature.getInstance(sigalgo, provider);

			}

		}
	}

	//		if (sigalgo.contains("EC")) {
	//			sig = Signature.getInstance(sigalgo, "FlexiEC");
	//		} else if (sigalgo.contains("IQ")) {
	//			sig = Signature.getInstance(sigalgo, "FlexiNF");
	//		} else {
	//			sig = Signature.getInstance(sigalgo, "FlexiCore");
	//		}
	//
	//	}

	public byte[] digest(byte[] encoded) {
		digest.reset();
		digest.update(encoded);
		return digest.digest();
	}

	public byte[] digest(String source) throws NoSuchAlgorithmException, NoSuchProviderException {
		digest.reset();
		digest.update(source.getBytes());
		return digest.digest();
	}

	public byte[] getSignature(PrivateKey pk, byte[] source) throws InvalidKeyException, SignatureException {
		sig.initSign(pk);
		sig.update(source);
		return sig.sign();
	}

	public void sign(PrivateKey priv, PublicKey pub, X509Certificate xc) throws InvalidKeyException,
			CertificateEncodingException, SignatureException, NoSuchAlgorithmException,
			InvalidAlgorithmParameterException {

		sig.initSign(priv);
		xc.sign(sig, pub);

	}

}
