package minica.crypto;

import java.security.InvalidAlgorithmParameterException;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;

import minica.Application;
import de.flexiprovider.ec.ecparameters.CurveParams;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This Class is responsible for the Key Generation by Flexiprovider.
 * 
 * @author stephan
 *
 */
public class FlexiKeyGenerator {

	private KeyPairGenerator	kpg;

	/**
	 * 
	 * @param algname Name of the algorithm
	 * @param keysize Key Length
	 * @param specs Key Generator Algorithm Parameter Specs, only for EC right now
	 * @throws NoSuchAlgorithmException
	 * @throws NoSuchProviderException
	 * @throws InvalidAlgorithmParameterException
	 */

	public FlexiKeyGenerator(String algname, int keysize, CurveParams specs) throws NoSuchAlgorithmException,
			NoSuchProviderException, InvalidAlgorithmParameterException {

		//System.out.println(algname);

		String[] algos = Application.getSettings().getArray("WorkingAlgorithms");
		for (int i = 0; i < algos.length; i++) {
			String current = algos[i];

			int cut = current.indexOf(",");
			int cut2 = current.indexOf(",", cut + 1);
			String algo = current.substring(0, cut);
			String provider = current.substring(cut + 1, cut2);
			String params = current.substring(cut2 + 1, current.length());
			//System.out.println("Keygen: " + algo + " " + provider + " " + params);
			if (algo.equals(algname)) {
				if (params.equals("true")) {
					SecureRandom random = SecureRandom.getInstance("BBSRandom", "FlexiCore");
					kpg = KeyPairGenerator.getInstance(algname, provider);
					kpg.initialize(specs, random);
				} else {
					kpg = KeyPairGenerator.getInstance(algname, provider);
					kpg.initialize(keysize);
				}

			}
		}

		//		if (algname.contains("EC")) {
		//			//System.out.println("EC Zweig");
		//			SecureRandom random = SecureRandom.getInstance("BBSRandom", "FlexiCore");
		//			kpg = KeyPairGenerator.getInstance(algname, "FlexiEC");
		//			kpg.initialize(specs, random);
		//			//kpg.initialize(keysize);
		//		} else if (algname.contains("IQ")) {
		//			kpg = KeyPairGenerator.getInstance(algname, "FlexiNF");
		//			kpg.initialize(keysize);
		//		} else if (algname.equals("RSA") || algname.equals("DSA")) {
		//			kpg = KeyPairGenerator.getInstance(algname, "FlexiCore");
		//			kpg.initialize(keysize);
		//		} else {
		//			kpg = KeyPairGenerator.getInstance(algname, "FlexiCore");
		//			kpg.initialize(keysize);
		//		}
	}

	public KeyPair createKey() {
		KeyPair kp = kpg.generateKeyPair();
		return kp;
	}
}
