package minica.crypto;

import java.math.BigInteger;
import java.util.List;

import codec.x509.X509Certificate;
import codec.x509.X509Crl;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * An Interface for a (any) CA Certificate, either Root-CA or
 * subsidary CA. Holds any related information, like sub-certs,
 * CRLs, information, settings, keys, etc.
 * TODO add information
 * Notice: Every implementing class _must_ properly implement hashCode and equals!!
 * @author tex
 *
 */
public interface ICaCertificate extends ICertificate {
	public boolean addCert(ICertificate cert);

	// does the list contain the given object? (needed for TreeContentAdapter)
	public boolean contains(Object o);

	public ICaCertificate get(Object o);

	/**
	 * Get all subcerts under this ca. does a BFS.
	 * @return
	 */
	public List<ICaCertificate> getAllCaCerts();

	public List<ICaCertificate> getCaCerts();

	// this ca's certificate
	public X509Certificate getCertificate();

	public List<ICertificate> getCertificates();

	// direct client certificates
	public IClientCertificate[] getClientCertificates();

	public List<IClientCertificate> getClientCerts();

	public X509Crl getCRL();

	public X509Crl getDeltaCrl();

	// null if this a root-ca.
	public ICaCertificate getParentCa();

	public ICertificate getSubCert(BigInteger serial);

	// sub-ca's
	public ICaCertificate[] getSubCertificates();

	public boolean removeCert(ICertificate cert);

	public void setCertificate(X509Certificate cert);

	public void setCRL(X509Crl crl);

	public void setDeltaCrl(X509Crl deltaCrl);

	//get specific cert

	public void setParentCa(ICaCertificate parent);

}
