package minica.crypto;

import java.util.List;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This class will act as the connection between the main keyStore (for 
 * storing certificates and keys, as needed) and the Tree GUI component.
 * @author tex
 * @see http://www.eclipse.org/articles/Article-TreeViewer/TreeViewerArticle.htm#mainView <-- guter Artikel :)
 */
public class KeyStoreTreeContentAdapter implements ITreeContentProvider {

	private KeyStoreProvider	keystore;

	public KeyStoreTreeContentAdapter() {
		this.keystore = KeyStoreProvider.getInstance();
	}

	public void dispose() {
		// NO-OP
	}

	public Object[] getChildren(Object arg0) {
		// Wichtig!!!!
		if (this.keystore.equals(arg0)) {
			return this.keystore.getCertificates().toArray();
		}
		List<ICaCertificate> certs = this.keystore.getCertificates();
		for (int i = 0; i < certs.size(); i++) {
			if (certs.get(i).contains(arg0)) {
				return certs.get(i).get(arg0).getCertificates().toArray();
			}
		}
		// if nothing is found, return null
		return new Object[0];
	}

	public Object[] getElements(Object arg0) {
		return getChildren(arg0);
	}

	public Object getParent(Object arg0) {
		if (this.keystore.equals(arg0)) {
			return null;
		}
		List<ICaCertificate> certs = this.keystore.getCertificates();
		for (int i = 0; i < certs.size(); i++) {
			if (certs.get(i).contains(arg0)) {
				return certs.get(i).get(arg0).getParent();
			}
		}
		// if nothing is found, return null
		return null;
	}

	public boolean hasChildren(Object arg0) {
		if (this.keystore.equals(arg0)) {
			if (this.keystore.getCertificates().size() > 0) {
				return true;
			} else {
				return false;
			}
		}

		List<ICaCertificate> certs = this.keystore.getCertificates();
		for (int i = 0; i < certs.size(); i++) {
			ICaCertificate cur = certs.get(i);
			if (certs != null && cur != null && cur.contains(arg0)) {
				if (certs.get(i).get(arg0) != null && certs.get(i).get(arg0).getCertificates() != null
						&& certs.get(i).get(arg0).getCertificates().size() > 0) {
					return true;
				} else {
					return false;
				}
			}
		}
		// if nothing is found, return false
		return false;
	}

	public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
		// NO-OP
	}

}
