package minica.crypto;

import java.util.ArrayList;
import java.util.List;

import minica.Activator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.progress.UIJob;

import codec.x509.X509Crl;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * The Label Adapter for the Data struct.
 * @author tex
 *
 */
public class KeyStoreTreeLabelAdapter implements ILabelProvider, IModelChangeListener {

	private List<ILabelProviderListener>	listener;

	private ILabelProviderListener			currentListener;

	public KeyStoreTreeLabelAdapter() {
		this.listener = new ArrayList<ILabelProviderListener>();
		KeyStoreProvider ksp = KeyStoreProvider.getInstance();
		ksp.addChangeListener(this);
	}

	public void addListener(ILabelProviderListener arg0) {
		this.listener.add(arg0);
	}

	public void dispose() {
		//NO-OP
	}

	public Image getImage(Object arg0) {
		Image i = Activator.getImageDescriptor("icons/ok.gif").createImage();
		// determine the apt icon and display it.
		if (arg0 instanceof ICaCertificate) {
			return Activator.getImageDescriptor("icons/page_white_key.png").createImage();
		} else if (arg0 instanceof IClientCertificate) {
			return Activator.getImageDescriptor("icons/script_edit.png").createImage();
		} else if (arg0 instanceof X509Crl) {
			return Activator.getImageDescriptor("icons/script_error.png").createImage();
		}
		return i;
	}

	public String getText(Object arg0) {
		// determine the apt text and display it.
		if (arg0 instanceof ICertificate) {
			ICertificate cert = (ICertificate) arg0;
			return cert.getCertificate().getSubjectX500Principal().getName() + " ("
					+ String.valueOf(cert.getCertificate().getSerialNumber()) + ")";
		}
		return null;
	}

	public boolean isLabelProperty(Object arg0, String arg1) {
		// Not implemented
		return false;
	}

	public void refresh() {
		//System.out.println("KeyStoreTreeLabelAdapter.refresh()");
		for (ILabelProviderListener l : this.listener) {
			//l.labelProviderChanged(e);
			this.currentListener = l;
			UIJob uijob = new UIJob("Refreshing Tree") {
				@Override
				public IStatus runInUIThread(IProgressMonitor monitor) {
					//System.out.println("KeyStoreTreeLabelAdapter.refresh() - Refreshing tree!");
					LabelProviderChangedEvent e = new LabelProviderChangedEvent(KeyStoreTreeLabelAdapter.this);
					KeyStoreTreeLabelAdapter.this.currentListener.labelProviderChanged(e);
					return Status.OK_STATUS;
				}
			};
			uijob.schedule();
		}
	}

	public void removeListener(ILabelProviderListener arg0) {
		this.listener.remove(arg0);
	}

}
