package minica.crypto;

import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.security.Security;
import java.security.Signature;
import java.security.Provider.Service;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import codec.x509.AlgorithmIdentifier;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * The signature detection job.
 * @author tex
 *
 */
public class SignatureDetectionJob extends Job {

	public SignatureDetectionJob(String name) {
		super(name);
		// TODO Auto-generated constructor stub
	}

	private boolean isValidSignAlgo(String algorithm) {
		if (!algorithm.contains("with")) {
			return false;
		}
		try {
			@SuppressWarnings("unused")
			Signature s = Signature.getInstance(algorithm);
			@SuppressWarnings("unused")
			AlgorithmIdentifier ai = new AlgorithmIdentifier(algorithm);
		} catch (NoSuchAlgorithmException e) {
			return false;
		}
		return true;
	}

	@Override
	protected IStatus run(IProgressMonitor arg0) {
		CryptoInformationProvider cip = CryptoInformationProvider.getInstance();
		Provider[] providers = Security.getProviders();
		for (int i = 0; i < providers.length; i++) {
			Set<Service> services = providers[i].getServices();
			for (Service s : services) {
				if (s.getType().equals("Signature")) {
					if (this.isValidSignAlgo(s.getAlgorithm())) {
						cip.addValidSignatureAlgorithm(s.getAlgorithm());
					}
				}
			}
		}
		//System.out.println("DEBUG: SignatureDetectionJob finished");
		return Status.OK_STATUS;
	}

}
