package minica.util;

import java.text.DateFormat;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;

import codec.asn1.ASN1BitString;
import codec.asn1.ASN1Integer;
import codec.asn1.ASN1OctetString;
import codec.asn1.ASN1Sequence;
import codec.x509.X509Certificate;
import codec.x509.X509Extension;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This is a Helper Class. It is used to parse certain fields of a Certificate.
 * @author ???
 *
 */
public class CertificateHelper {

	private X509Certificate				certificate;

	private Collection<X509Extension>	extensions;

	@SuppressWarnings("unchecked")
	public CertificateHelper(X509Certificate xc) {
		this.certificate = xc;
		this.extensions = xc.getExtensions();
	}

	public String getAKIE() {

		Iterator<X509Extension> iterator = extensions.iterator();
		while (iterator.hasNext()) {
			X509Extension extension = iterator.next();
			String oid = extension.getOID().toString();
			if (oid.equals("2.5.29.35")) {

				ASN1Sequence akie = (ASN1Sequence) extension.getValue();
				String value = akie.toString();
				int begin = value.indexOf("String") + 7;
				int end = begin + 60;
				return value.substring(begin, end);

			}
		}
		return "-1";
	}

	public X509Certificate getCertificate() {
		return certificate;
	}

	public String[] getIssuerDN_array() {
		String temp = certificate.getIssuerDN().getName();
		String[] dn = new String[4];
		dn[0] = temp.substring(temp.indexOf("CN=") + 3, temp.indexOf(", OU="));
		dn[1] = temp.substring(temp.indexOf("O=") + 2, temp.indexOf(", C="));
		dn[2] = temp.substring(temp.indexOf("OU=") + 3, temp.indexOf(", O="));
		dn[3] = temp.substring(temp.indexOf("C=") + 2, temp.length());
		return dn;
	}

	public int[] getKeyUsage_int() {

		Iterator<X509Extension> iterator = extensions.iterator();
		while (iterator.hasNext()) {
			X509Extension extension = iterator.next();
			String oid = extension.getOID().toString();
			if (oid.equals("2.5.29.15")) {
				int[] values = new int[9];
				boolean[] temp = ((ASN1BitString) extension.getValue()).getBits();
				int length = temp.length;
				for (int i = 0; i < 9; i++) {
					values[i] = 0;
					if (i < length) {
						if (temp[i])
							values[i] = 1;
					}
				}
				return values;
			}
		}
		return null;
	}

	public String getPathLength() {

		Iterator<X509Extension> iterator = extensions.iterator();
		while (iterator.hasNext()) {
			X509Extension extension = iterator.next();
			String oid = extension.getOID().toString();
			if (oid.equals("2.5.29.19")) {
				return ((ASN1Integer) ((ASN1Sequence) extension.getValue()).get(1)).getBigInteger().toString();
			}
		}
		return "-1";

	}

	public String getSKIE() {

		Iterator<X509Extension> iterator = extensions.iterator();
		while (iterator.hasNext()) {
			X509Extension extension = iterator.next();
			String oid = extension.getOID().toString();
			if (oid.equals("2.5.29.14")) {
				ASN1OctetString skie = (ASN1OctetString) extension.getValue();
				return (skie.toString().replace("Octet String ", ""));
			}
		}
		return "-1";
	}

	public String[] getSubjectDN_array() {
		String temp = certificate.getSubjectDN().getName();
		String[] dn = new String[4];
		dn[0] = temp.substring(temp.indexOf("CN=") + 3, temp.indexOf(", OU="));
		dn[1] = temp.substring(temp.indexOf("O=") + 2, temp.indexOf(", C="));
		dn[2] = temp.substring(temp.indexOf("OU=") + 3, temp.indexOf(", O="));
		dn[3] = temp.substring(temp.indexOf("C=") + 2, temp.length());
		return dn;
	}

	public String[] getValidity() {
		String notbefore = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.GERMANY).format(
				certificate.getNotBefore());
		String notafter = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.GERMANY).format(
				certificate.getNotAfter());
		String[] validity = { notbefore, notafter };
		return validity;
	}

}
