package minica.views;

import minica.crypto.ICertificate;
import minica.crypto.KeyStoreProvider;
import minica.crypto.KeyStoreTreeContentAdapter;
import minica.crypto.KeyStoreTreeLabelAdapter;
import minica.crypto.ModelChangeListener;
import minica.editors.CertificateMultiPageEditor;
import minica.editors.CertificateMultiPageEditorInput;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * TODO Class description
 * @author ???
 * @author dominik
 *
 */
public class CertificateTreeView extends ViewPart {

	public static final String	ID	= "minica.views.CertificateTreeView";

	private TreeViewer			treeViewer;

	public CertificateTreeView() {
		// We don't Overwrite CertificateTreeView Constructor to use real Data Model, do we?
		super();
	}

	@Override
	public void createPartControl(Composite parent) {
		treeViewer = new TreeViewer(parent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		KeyStoreProvider.getInstance().addChangeListener(new ModelChangeListener(treeViewer));
		getSite().setSelectionProvider(treeViewer);
		this.initialize();
		// Changed Code to acquire data from Model
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				// NO-OP
			}
		});
		treeViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				IWorkbenchPage x = getSite().getPage();
				try {
					IStructuredSelection selection = (IStructuredSelection) treeViewer.getSelection();
					x.openEditor(new CertificateMultiPageEditorInput((ICertificate) selection.getFirstElement()),
							CertificateMultiPageEditor.ID);
				} catch (PartInitException e) {
					e.printStackTrace();
				}
			}
		});
		KeyStoreProvider ksp = KeyStoreProvider.getInstance();
		treeViewer.setInput(ksp);
		treeViewer.expandAll();
	}

	public void initialize() {
		treeViewer.setContentProvider(new KeyStoreTreeContentAdapter());
		treeViewer.setLabelProvider(new KeyStoreTreeLabelAdapter());

	}

	@Override
	public void setFocus() {
		treeViewer.getControl().setFocus();
	}

}
