package minica.wizards;

import java.math.BigInteger;
import java.security.Principal;
import java.util.ArrayList;
import java.util.GregorianCalendar;

import minica.crypto.ICaCertificate;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This class encapsulates all the data needed to create a (Delta)CRL.
 * @author stephan
 *
 */

public class CRLJobDescription {

	public final static int			CRL						= 0;

	public final static int			DELTACRL				= 1;

	private String					filename;

	private int						type;

	private boolean					complete				= false;

	private ICaCertificate			issuer;

	private ArrayList<BigInteger>	toBeRevokedCertificates	= new ArrayList<BigInteger>();

	private ArrayList<Integer>		reasonCodes				= new ArrayList<Integer>();

	private GregorianCalendar		nextUpdate;

	private String					issuerPassword;

	// Setters/Getters

	public void addToBeRevokedCertificate(BigInteger cert, Integer reason) {
		toBeRevokedCertificates.add(cert);
		reasonCodes.add(reason);
	}

	public String getFileName() {
		return filename;
	}

	public ICaCertificate getIssuer() {
		return issuer;
	}

	public Principal getIssuerDN() {
		return issuer.getCertificate().getSubjectDN();
	}

	public String getIssuerPassword() {
		return issuerPassword;
	}

	public GregorianCalendar getNextUpdate() {
		return nextUpdate;
	}

	public ArrayList<Integer> getReasonCodes() {
		return reasonCodes;
	}

	public ArrayList<BigInteger> getToBeRevokedCertificates() {
		return toBeRevokedCertificates;
	}

	public int getType() {
		return type;
	}

	public boolean isComplete() {
		return complete;
	}

	public void removeToBeRevokedCertificate(BigInteger cert) {
		int i = toBeRevokedCertificates.indexOf(cert);
		toBeRevokedCertificates.remove(i);
		reasonCodes.remove(i);
	}

	public void setComplete(boolean complete) {
		this.complete = complete;
	}

	public void setFileName(String text) {
		filename = text;
	}

	public void setIssuer(ICaCertificate issuer) {
		this.issuer = issuer;
	}

	public void setIssuerPassword(String issuerPassword) {
		this.issuerPassword = issuerPassword;
	}

	public void setNextUpdate(GregorianCalendar cal) {
		this.nextUpdate = cal;
	}

	public void setType(int arg) {
		type = arg;
	}
}
