package minica.wizards;

import java.security.GeneralSecurityException;
import java.security.KeyPair;

import minica.crl.CreateCRLJob;
import minica.crypto.KeyStoreProvider;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.wizard.Wizard;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This class represents a CRL Wizard.
 * @author ???
 *
 */
public class CRLWizard extends Wizard {
	private CRLJobDescription	jobdescription;

	public CRLWizard() {
		setNeedsProgressMonitor(false);
		jobdescription = new CRLJobDescription();
		setWindowTitle("Create CRL");
		addPage(new CRLWizardFrontPage("CRL Wizard", jobdescription));
		addPage(new CRLWizardPropertiesPage("CRL Wizard", jobdescription));
	}

	@Override
	public boolean canFinish() {
		return jobdescription.isComplete();
	}

	@Override
	public boolean performFinish() {
		
			try {
				@SuppressWarnings("unused")
				KeyPair kp = KeyStoreProvider.getInstance()
					.getCertificate(jobdescription.getIssuer().getCertificate().getSerialNumber())
					.getKeyPair(jobdescription.getIssuerPassword());
			} catch (GeneralSecurityException e) {
		        Status status = new Status(IStatus.ERROR, "CRLWizard", 0,
		                "GeneralSecurityException caught!", null);
		            // Display the dialog
		            ErrorDialog.openError(getShell(),
		                "Error", "PKCS#12 Password wrong?", status);
				return true;
			}
		
		CreateCRLJob crljob = new CreateCRLJob(jobdescription);
		crljob.schedule();
		return true;
	}

}
