package minica.wizards;

import java.math.BigInteger;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import minica.crypto.KeyStoreProvider;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This is the CRL Wizard Front Page
 * @author stephan
 *
 */
public class CRLWizardFrontPage extends WizardPage {

	private CRLJobDescription	jobdescription;

	private Text				textFilename;

	private Combo				comboIssuer;

	private Combo				comboCRLType;

	private boolean				fileNameSet	= false;

	private boolean				issuerSet	= false;

	private boolean				typeSet		= false;

	private boolean				passwordSet	= false;

	private Combo				comboDayStart;

	private Combo				comboYearStart;

	private Combo				comboMonthStart;

	private final String[]		Months		= new String[] { "January", "February", "March", "April", "May", "June",
			"July", "August", "September", "October", "November", "December" };

	private GregorianCalendar	calendarStart;

	private Text				textPassword;

	protected CRLWizardFrontPage(String pageName) {
		super(pageName);
	}

	protected CRLWizardFrontPage(String name, CRLJobDescription description) {
		super(name);
		jobdescription = description;
		setTitle("Create CRL Wizard 1/2");
		setDescription("Use this Wizard to create a CRL");
		// FIXME display bug when more than about 20 certificates. try it out.
		setPageComplete(false);
	}

	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.None);
		composite.setLayout(new GridLayout(1, false));

		Group groupFilename = new Group(composite, SWT.None);
		groupFilename.setLayout(new GridLayout(2, false));
		groupFilename.setText("Filename");
		groupFilename.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelFilename = new Label(groupFilename, SWT.None);
		labelFilename.setText("Filename:");

		Composite compositeTextFilename = new Composite(groupFilename, SWT.NULL);
		compositeTextFilename.setLayout(new GridLayout(1, false));
		compositeTextFilename.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		textFilename = new Text(compositeTextFilename, SWT.BORDER);
		textFilename.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Group groupCRLSettings = new Group(composite, SWT.None);
		groupCRLSettings.setLayout(new GridLayout(2, false));
		groupCRLSettings.setText("CRL Settings");
		groupCRLSettings.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelIssuer = new Label(groupCRLSettings, SWT.None);
		labelIssuer.setText("Issuer:");

		Composite composite1 = new Composite(groupCRLSettings, SWT.NULL);
		composite1.setLayout(new GridLayout(1, false));
		composite1.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		comboIssuer = new Combo(composite1, SWT.READ_ONLY);
		comboIssuer.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		//Fill Serials
		int size = KeyStoreProvider.getInstance().getCaCertificateSerials().size();
		List<BigInteger> serials = KeyStoreProvider.getInstance().getCaCertificateSerials();
		for (int i = 0; i < size; i++) {
			comboIssuer.add(serials.get(i).toString());
		}

		Label labelPassword = new Label(groupCRLSettings, SWT.None);
		labelPassword.setText("Issuer Password:");

		Composite compositePassword = new Composite(groupCRLSettings, SWT.NULL);
		compositePassword.setLayout(new GridLayout(1, false));
		compositePassword.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		textPassword = new Text(compositePassword, SWT.PASSWORD);
		textPassword.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelCRLType = new Label(groupCRLSettings, SWT.None);
		labelCRLType.setText("CRL Type:");

		Composite composite2 = new Composite(groupCRLSettings, SWT.NULL);
		composite2.setLayout(new GridLayout(1, false));
		composite2.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		comboCRLType = new Combo(composite2, SWT.READ_ONLY);
		comboCRLType.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
		comboCRLType.add("CRL");
		comboCRLType.add("Delta-CRL");

		Group groupValidityPeriod = new Group(composite, SWT.NULL);
		groupValidityPeriod.setLayout(new GridLayout(6, false));
		groupValidityPeriod.setText("Validity Period");
		groupValidityPeriod.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelDayStart = new Label(groupValidityPeriod, SWT.NULL);
		labelDayStart.setText("Not Before: Day:");
		comboDayStart = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboDayStart.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelMonthStart = new Label(groupValidityPeriod, SWT.NULL);
		labelMonthStart.setText("Month:");
		comboMonthStart = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboMonthStart.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelYearStart = new Label(groupValidityPeriod, SWT.NULL);
		labelYearStart.setText("Year:");
		comboYearStart = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboYearStart.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		initCalendar();

		SelectionListener DateCheckListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				int month_start = comboMonthStart.getSelectionIndex();
				int year_start = new Integer(comboYearStart.getText()).intValue();
				int day_start = comboDayStart.getSelectionIndex() + 1;

				int month_end = comboMonthStart.getSelectionIndex();

				GregorianCalendar tempcal = new GregorianCalendar();
				tempcal.set(Calendar.DATE, 1);
				tempcal.set(Calendar.MONTH, month_start);
				tempcal.set(Calendar.YEAR, year_start);

				int max_day = tempcal.getActualMaximum(Calendar.DATE);

				comboDayStart.removeAll();

				for (int i = 1; i < max_day + 1; i++) {
					comboDayStart.add(((Integer) i).toString());
				}

				if (day_start > max_day) {
					day_start = max_day;
					comboDayStart.select(max_day - 1);
				} else {
					comboDayStart.select(day_start - 1);
				}

				calendarStart.set(year_start, month_start, day_start);

				tempcal.set(Calendar.DATE, 1);
				tempcal.set(Calendar.MONTH, month_end);

				max_day = tempcal.getActualMaximum(Calendar.DATE);

				checkFinished();

			}

		};

		comboDayStart.addSelectionListener(DateCheckListener);
		comboMonthStart.addSelectionListener(DateCheckListener);
		comboYearStart.addSelectionListener(DateCheckListener);

		Listener textBoxListener = new Listener() {

			public void handleEvent(Event event) {

				String filename = textFilename.getText();
				if (!filename.equals("")) {
					setErrorMessage(null);
					fileNameSet = true;
				} else {
					setErrorMessage("Please insert a Filename!");
					fileNameSet = false;
				}

				if (!textPassword.equals("")) {
					setErrorMessage(null);
					passwordSet = true;
				} else {
					setErrorMessage("Please insert the Issuer PKCS#12 Password!");
					passwordSet = false;
				}
				checkFinished();
			}

		};

		SelectionListener ComboSelectionListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			public void widgetSelected(SelectionEvent arg0) {

				if ((comboCRLType.getText()) != "") {
					typeSet = true;
				}
				if ((comboIssuer.getText() != "")) {
					issuerSet = true;
				}
				checkFinished();
			}

		};

		comboCRLType.addSelectionListener(ComboSelectionListener);
		comboIssuer.addSelectionListener(ComboSelectionListener);

		textFilename.addListener(SWT.KeyUp, textBoxListener);
		textPassword.addListener(SWT.KeyUp, textBoxListener);

		setControl(composite);

	}

	public void initCalendar() {

		calendarStart = new GregorianCalendar();
		calendarStart.set(GregorianCalendar.MONTH, calendarStart.get(GregorianCalendar.MONTH + 3));

		int currentYear = calendarStart.get(GregorianCalendar.YEAR);
		int currentMonth = calendarStart.get(GregorianCalendar.MONTH);
		int currentDay = calendarStart.get(GregorianCalendar.DATE);

		comboMonthStart.setItems(Months);
		comboMonthStart.select(currentMonth);

		for (int i = currentYear - 20; i < currentYear + 20; i++) {
			comboYearStart.add(((Integer) i).toString());
		}
		comboYearStart.select(19);

		int maxDaysStart = calendarStart.getActualMaximum(GregorianCalendar.DATE);

		for (int i = 1; i < maxDaysStart + 1; i++) {
			comboDayStart.add(((Integer) i).toString());
		}

		comboDayStart.select(currentDay - 1);

	}

	private void checkFinished() {
		if (fileNameSet && typeSet && issuerSet && passwordSet) {

			jobdescription.setFileName(textFilename.getText());
			jobdescription.setIssuerPassword(textPassword.getText());
			jobdescription.setIssuer(KeyStoreProvider.getInstance().getCertificate(
					new BigInteger(comboIssuer.getText())));
			if (comboCRLType.getText().equals("Delta-CRL")) {
				jobdescription.setType(CRLJobDescription.DELTACRL);
			} else {
				jobdescription.setType(CRLJobDescription.CRL);
			}
			jobdescription.setNextUpdate(calendarStart);
			setPageComplete(true);
		} else {
			setPageComplete(false);
		}

	}

}
