package minica.wizards;

import java.math.BigInteger;
import java.util.ArrayList;

import minica.crypto.ICaCertificate;
import minica.crypto.ICertificate;
import minica.util.CRLHelper;
import minica.util.CertificateHelper;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.progress.UIJob;

import codec.x509.X509Certificate;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This is the Properties Page of the CRL Wizard.
 * 
 * @author stephan
 *
 */

public class CRLWizardPropertiesPage extends WizardPage {

	private CRLJobDescription				jobdescription;

	private List							listCertificates;

	private String[]						reasons				= { "unspecified", "Key Compromised", "CA Compromised",
			"Affiliation Changed", "Superseded", "Cessation of Operation", "Certificate Hold", "Remove from CRL",
			"Privilege Withdrawn", "aACompromise"				};

	private List							listRevokedCertificates;

	private Text							textSerial;

	private Text							textOU;

	private Text							textO;

	private Text							textCN;

	private Text							textC;

	private java.util.List<ICertificate>	certs;

	private int								certSelected		= 0;

	private Combo							comboReason;

	private ArrayList<BigInteger>			revokedCertificates;

	private Button							buttonRevoke;

	private boolean							enableRevokeButton	= true;

	protected CRLWizardPropertiesPage(String pageName) {
		super(pageName);

	}

	protected CRLWizardPropertiesPage(String pageName, CRLJobDescription description) {
		super(pageName);
		jobdescription = description;
		setTitle("Create CRL Wizard 2/2");
		setDescription("Use this Wizard to create a CRL");
	}

	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.None);
		composite.setLayout(new GridLayout(1, false));

		Composite innerComposite = new Composite(composite, SWT.None);
		innerComposite.setLayout(new GridLayout(3, false));
		innerComposite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelCertificates = new Label(innerComposite, SWT.None);
		labelCertificates.setText("Valid Certificates");
		labelCertificates.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		@SuppressWarnings("unused")
		Label placeholder = new Label(innerComposite, SWT.None);

		Label labelCertificatesToRevoke = new Label(innerComposite, SWT.None);
		labelCertificatesToRevoke.setText("Revoked Certificates");
		labelCertificatesToRevoke
				.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		listCertificates = new List(innerComposite, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		listCertificates.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Composite compositeButtons = new Composite(innerComposite, SWT.None);
		compositeButtons.setLayout(new GridLayout(1, false));
		Button buttonToRight = new Button(compositeButtons, SWT.NONE);
		buttonRevoke = new Button(compositeButtons, SWT.NONE);
		buttonToRight.setText(">");
		buttonRevoke.setText("<");

		listRevokedCertificates = new List(innerComposite, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		listRevokedCertificates.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Composite compositeReason = new Composite(composite, SWT.NONE);
		compositeReason.setLayout(new GridLayout(2, false));
		compositeReason.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelReason = new Label(compositeReason, SWT.None);
		labelReason.setText("Reason:");

		comboReason = new Combo(compositeReason, SWT.READ_ONLY);
		comboReason.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		for (int i = 0; i < reasons.length; i++) {
			comboReason.add(reasons[i]);
		}

		Group groupCertificateInformation = new Group(composite, SWT.NONE);
		groupCertificateInformation.setLayout(new GridLayout(2, false));
		groupCertificateInformation.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.GRAB_HORIZONTAL));
		groupCertificateInformation.setText("Certificate Information");

		Label labelCN = new Label(groupCertificateInformation, SWT.NONE);
		labelCN.setText("Common Name:");
		textCN = new Text(groupCertificateInformation, SWT.READ_ONLY | SWT.BORDER);
		textCN.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelO = new Label(groupCertificateInformation, SWT.NONE);
		labelO.setText("Organisation:");
		textO = new Text(groupCertificateInformation, SWT.READ_ONLY | SWT.BORDER);
		textO.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelOU = new Label(groupCertificateInformation, SWT.NONE);
		labelOU.setText("Organisational Unit:");
		textOU = new Text(groupCertificateInformation, SWT.READ_ONLY | SWT.BORDER);
		textOU.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelC = new Label(groupCertificateInformation, SWT.NONE);
		labelC.setText("Organisational Unit:");
		textC = new Text(groupCertificateInformation, SWT.READ_ONLY | SWT.BORDER);
		textC.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelSerial = new Label(groupCertificateInformation, SWT.NONE);
		labelSerial.setText("Serial Number:");
		textSerial = new Text(groupCertificateInformation, SWT.READ_ONLY | SWT.BORDER);
		textSerial.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		// Listener to check if a Certificate in RevokedCertificates can be removed

		SelectionListener revocationListSelectionListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				setErrorMessage(null);
				X509Certificate cert = null;
				listCertificates.deselectAll();
				if (listRevokedCertificates.getSelectionCount() > 0) {
					BigInteger i = new BigInteger(listRevokedCertificates.getSelection()[0]);
					cert = (X509Certificate) jobdescription.getIssuer().getSubCert(i).getCertificate();
					CertificateHelper helper = new CertificateHelper(cert);
					String[] dn = helper.getSubjectDN_array();
					textCN.setText(dn[0]);
					textO.setText(dn[1]);
					textOU.setText(dn[2]);
					textC.setText(dn[3]);
					textSerial.setText(cert.getSerialNumber().toString());

					for (String current : listRevokedCertificates.getSelection()) {
						setErrorMessage(null);
						BigInteger currentCertSerial = new BigInteger(current);
						enableRevokeButton = true;
						for (int j = 0; j < revokedCertificates.size(); j++) {
							if (revokedCertificates.get(j).equals(currentCertSerial)) {
								//System.out.println("disable button");
								enableRevokeButton = false;

							}

						}

						UIJob uijob = new UIJob("Switch Button") {

							@Override
							public IStatus runInUIThread(IProgressMonitor monitor) {
								buttonRevoke.setEnabled(enableRevokeButton);
								return Status.OK_STATUS;
							}

						};
						uijob.schedule();

					}
				}

			}

		};

		//Listener for Certificate List Selection

		SelectionListener certificateListSelectionListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				//do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				setErrorMessage(null);
				X509Certificate cert = null;
				listRevokedCertificates.deselectAll();
				//System.out.println("listCertificates");
				if (listCertificates.getSelectionCount() > 0) {
					BigInteger i = new BigInteger(listCertificates.getSelection()[0]);
					cert = (X509Certificate) jobdescription.getIssuer().getSubCert(i).getCertificate();
					CertificateHelper helper = new CertificateHelper(cert);
					String[] dn = helper.getSubjectDN_array();
					textCN.setText(dn[0]);
					textO.setText(dn[1]);
					textOU.setText(dn[2]);
					textC.setText(dn[3]);
					textSerial.setText(cert.getSerialNumber().toString());
				}
			}
		};

		//Listeners for < / > Buttons

		buttonToRight.addListener(SWT.MouseDown, new Listener() {

			public void handleEvent(Event event) {
				setErrorMessage(null);
				String[] selection = listCertificates.getSelection();
				if (listCertificates.getSelectionCount() > 0) {

					if (comboReason.getSelectionIndex() != -1) {

						certSelected = certSelected + listCertificates.getSelectionCount();
						for (String cur : selection) {
							listCertificates.remove(cur);
							listRevokedCertificates.add(cur);
							jobdescription.addToBeRevokedCertificate(new BigInteger(cur), comboReason
									.getSelectionIndex());
						}

					} else {
						setErrorMessage("Please Select a Reason for the Revocation of this Certificate!");
					}
				}
				checkFinished();

			}

		});

		buttonRevoke.addListener(SWT.MouseDown, new Listener() {

			public void handleEvent(Event event) {
				setErrorMessage(null);
				String[] selection = listRevokedCertificates.getSelection();
				if (listRevokedCertificates.getSelectionCount() > 0) {
					certSelected = certSelected - listCertificates.getSelectionCount();
					for (String cur : selection) {
						listRevokedCertificates.remove(cur);
						listCertificates.add(cur);
						jobdescription.removeToBeRevokedCertificate(new BigInteger(cur));
					}
				}
				checkFinished();

			}

		});

		//Listener to remove Error Message when Issuer is selected

		comboReason.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				//do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				setErrorMessage(null);

			}

		});

		listCertificates.addSelectionListener(certificateListSelectionListener);
		listRevokedCertificates.addSelectionListener(revocationListSelectionListener);
		setControl(composite);

	}

	@Override
	public void setVisible(boolean visible) {

		super.setVisible(visible);
		listCertificates.removeAll();
		listRevokedCertificates.removeAll();

		ICaCertificate issuer = jobdescription.getIssuer();
		certs = jobdescription.getIssuer().getCertificates();

		CRLHelper helper = new CRLHelper(issuer.getCRL());
		revokedCertificates = helper.getCRLEntrySerials();

		// if a deltacrl was issued before
		if (issuer.getDeltaCrl() != null) {
			// add all certs to revokedCertificates
			helper = new CRLHelper(issuer.getDeltaCrl());
			revokedCertificates.addAll(helper.getCRLEntrySerials());
		}

		//Populate Lists
		for (ICertificate current : certs) {
			BigInteger currentSerial = current.getCertificate().getSerialNumber();
			if (revokedCertificates.contains(currentSerial)) {
				listRevokedCertificates.add(currentSerial.toString());
			} else {
				listCertificates.add(currentSerial.toString());
			}

		}

	}

	//check if Page is complete

	private void checkFinished() {

		if (certSelected != 0) {
			jobdescription.setComplete(true);
			setPageComplete(true);
		} else {
			jobdescription.setComplete(false);
			setPageComplete(false);
		}

	}

}
