package minica.wizards;

import java.math.BigInteger;
import java.security.GeneralSecurityException;
import java.security.KeyPair;

import minica.cert.CertificateJobDescription;
import minica.cert.CreateCertificateJob;
import minica.crypto.KeyStoreProvider;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.wizard.Wizard;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * The Class that represents an entire Certificate Wizard.
 * This is used for all Certificate Wizards in FlexiCA.
 * Contains one or more Pages. Is created from CertificateWizardFactory.
 * @author ???
 *
 */
public class CertificateWizard extends Wizard {

	private CertificateJobDescription		jobdescription;

	private CertificateWizardFrontPage		front		= null;

	private CertificateWizardPropertiesPage	properties	= null;

	private CertificateWizardKeyUsagePage	keyusage	= null;

	public CertificateWizard(CertificateWizardFrontPage frontpage, CertificateJobDescription jobdescription) {
		//super();
		front = frontpage;
		setNeedsProgressMonitor(false);
		addPage(frontpage);
		if (jobdescription.getType() == CertificateJobDescription.BatchMode) {
			setWindowTitle("Create Certificate (Batch Mode)");
		} else if (jobdescription.getType() == CertificateJobDescription.OneClick) {
			setWindowTitle("Create Certificate (One-Click)");
		}
		this.jobdescription = jobdescription;
	}

	public CertificateWizard(CertificateWizardFrontPage frontpage, CertificateWizardPropertiesPage propertiespage,
			CertificateJobDescription jobdescription) {
		//super();
		front = frontpage;
		properties = propertiespage;
		setNeedsProgressMonitor(false);
		addPage(frontpage);
		addPage(propertiespage);
		setWindowTitle("Create Certificate");
		this.jobdescription = jobdescription;
	}

	public CertificateWizard(CertificateWizardFrontPage frontpage, CertificateWizardPropertiesPage propertiespage,
			CertificateWizardKeyUsagePage keyusagepage, CertificateJobDescription jobdescription) {
		//super();
		front = frontpage;
		properties = propertiespage;
		keyusage = keyusagepage;
		setNeedsProgressMonitor(false);
		addPage(frontpage);
		addPage(propertiespage);
		addPage(keyusagepage);
		setWindowTitle("Create Certificate");
		this.jobdescription = jobdescription;
	}

	@Override
	public boolean canFinish() {
		return jobdescription.isComplete();
	}

	@Override
	public boolean performFinish() {
		front.pageComplete();
		
		//Check PKCS#12 Password
		
		if (jobdescription.getType() != CertificateJobDescription.RootCertificate &&
				jobdescription.getType() != CertificateJobDescription.OneClick ) {
			try {
				@SuppressWarnings("unused")
				KeyPair kp = KeyStoreProvider.getInstance()
					.getCertificate(new BigInteger(jobdescription.getIssuer()))
					.getKeyPair(jobdescription.getIssuerPassword());
			} catch (GeneralSecurityException e) {
		        Status status = new Status(IStatus.ERROR, "CertificateWizard", 0,
		                "GeneralSecurityException caught!", null);
		            // Display the dialog
		            ErrorDialog.openError(getShell(),
		                "Error", "PKCS#12 Password wrong?", status);
				return true;
			}
		}
		
		if (properties != null) {
			properties.pageComplete();
		}

		if (keyusage != null) {
			keyusage.pageComplete();
		}

		//System.out.println(jobdescription.toString());
		// moved to certificate job
		//jobdescription.setSerial(KeyStoreProvider.getInstance().getNextSerial());
		CreateCertificateJob job = new CreateCertificateJob(jobdescription);
		job.setUser(true);
		job.schedule();
		return true;
	}

}
