package minica.wizards;

import minica.cert.CertificateJobDescription;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This is a Factory to create all the different kinds of wizards needed by FlexiCA.
 * Singleton-Pattern, use .getInstance(...) to get the Factory.
 * @author stephan
 *
 */

public class CertificateWizardFactory {

	private static CertificateWizardFactory	instance	= null;

	public static CertificateWizardFactory getInstance() {
		if (null == instance) {
			instance = new CertificateWizardFactory();
		}
		return instance;
	}

	/**
	 * Create a new Batch Certificate Wizard
	 * @return the new wizard
	 */

	public CertificateWizard createBatchCertificateWizard() {
		CertificateJobDescription certificateJobDescription = new CertificateJobDescription(
				CertificateJobDescription.BatchMode);
		CertificateWizardFrontPage frontPage = new CertificateWizardFrontPage(
				"Create new User Certificate (Batch Mode) 1/1", certificateJobDescription);
		CertificateWizard wizard = new CertificateWizard(frontPage, certificateJobDescription);
		return wizard;
	}

	/**
	 * Create a new CA Certificate Wizard
	 * @return the new wizard
	 */

	public CertificateWizard createCACertificateWizard() {
		CertificateJobDescription certificateJobDescription = new CertificateJobDescription(
				CertificateJobDescription.CACertificate);
		CertificateWizardFrontPage frontPage = new CertificateWizardFrontPage("Create new CA Certificate 1/2",
				certificateJobDescription);
		CertificateWizardPropertiesPage propertiesPage = new CertificateWizardPropertiesPage(
				"Create new CA Certificate 2/2", certificateJobDescription);
		CertificateWizard wizard = new CertificateWizard(frontPage, propertiesPage, certificateJobDescription);
		return wizard;
	}

	/**
	 * Create a new One Click Certificate Wizard
	 * @return the new wizard
	 */

	public CertificateWizard createOneClickCertificateWizard() {
		CertificateJobDescription certificateJobDescription = new CertificateJobDescription(
				CertificateJobDescription.OneClick);
		CertificateWizardFrontPage frontPage = new CertificateWizardFrontPage(
				"Create new User Certificate (One Click Mode) 1/1", certificateJobDescription);
		CertificateWizard wizard = new CertificateWizard(frontPage, certificateJobDescription);
		return wizard;
	}

	/**
	 * Create a new Root Certificate Wizard
	 * @return the new wizard
	 */

	public CertificateWizard createRootCertificateWizard() {
		CertificateJobDescription certificateJobDescription = new CertificateJobDescription(
				CertificateJobDescription.RootCertificate);
		CertificateWizardFrontPage frontPage = new CertificateWizardFrontPage("Create new Root Certificate 1/2",
				certificateJobDescription);
		CertificateWizardPropertiesPage propertiesPage = new CertificateWizardPropertiesPage(
				"Create new Root Certificate 2/2", certificateJobDescription);
		CertificateWizard wizard = new CertificateWizard(frontPage, propertiesPage, certificateJobDescription);
		return wizard;
	}

	/**
	 * Create a new (End-Entity) Certificate Wizard
	 * @return the new wizard
	 */

	public CertificateWizard createUserCertificateWizard() {
		CertificateJobDescription certificateJobDescription = new CertificateJobDescription(
				CertificateJobDescription.ClientCertificate);
		CertificateWizardFrontPage frontPage = new CertificateWizardFrontPage("Create new User Certificate 1/3",
				certificateJobDescription);
		CertificateWizardPropertiesPage propertiesPage = new CertificateWizardPropertiesPage(
				"Create new User Certificate 2/3", certificateJobDescription);
		CertificateWizardKeyUsagePage keyusagepage = new CertificateWizardKeyUsagePage(
				"Create new User Certificate 3/3", certificateJobDescription);
		CertificateWizard wizard = new CertificateWizard(frontPage, propertiesPage, keyusagepage,
				certificateJobDescription);
		return wizard;
	}

}
