package minica.wizards;

import java.math.BigInteger;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import minica.Application;
import minica.cert.CertificateJobDescription;
import minica.crypto.CryptoInformationProvider;
import minica.crypto.KeyStoreProvider;
import minica.util.CertificateHelper;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Text;

import de.flexiprovider.ec.ecparameters.CurveParams;
import de.flexiprovider.ec.ecparameters.CurveRegistry;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */

/**
 * This is a Certificate Wizard Front Page.
 * All Certificate Wizards contain a different configuration of this Page.
 * @author ???
 */

public class CertificateWizardFrontPage extends WizardPage {

	private Text						textFileName;

	private Combo						comboIssuer;

	private Text						textIssuerPassword;

	private Combo						comboKeyAlgorithm;

	private Text						textPKCS12Password;

	private Text						textPKCS12Password2;

	private Combo						comboKeyLength;

	private Map<String, Set<Integer>>	asymAlgos;

	private Set<String>					signAlgos;

	private boolean						fileNameSet					= false;

	private boolean						issuerSelected				= false;

	private boolean						issuerPasswordSet			= false;

	private boolean						algorithmSelected			= false;

	private boolean						pkcs12PasswordSet			= false;

	private boolean						keyLengthSelected			= false;

	private boolean						signatureAlgorithmSelected	= false;

	private boolean						pathLengthValid				= true;

	private boolean						commonNameSet				= true;

	private Text						textCommonName;

	private CertificateJobDescription	certificateJobDescription;

	private Combo						comboSignatureAlgorithm;

	private Combo						comboCurve;

	private Label						labelKeyStrengthValue;

	private Scale						scale;

	private Text						textPW;

	/**
	 * Constructor for a new Frontpage
	 * @param pageName Name of the Page
	 * @param data a CertificateJobDescription with the type variable identifing the needed Page
	 */

	public CertificateWizardFrontPage(String pageName, CertificateJobDescription data) {

		super(pageName);

		certificateJobDescription = data;

		int type = certificateJobDescription.getType();

		if (type == CertificateJobDescription.CACertificate) {
			setTitle("Create new CA Certificate 1/2");
			setDescription("Use this Wizard to create a new CA Certificate");
		} else if (type == CertificateJobDescription.ClientCertificate) {
			setTitle("Create new User Certificate 1/3");
			setDescription("Use this Wizard to create a new User Certificate");
		} else if (type == CertificateJobDescription.BatchMode) {
			setTitle("Create new User Certificates 1/1");
			setDescription("Use this Wizard to create several new User Certificates in Batch Mode");
		} else if (type == CertificateJobDescription.OneClick) {
			setTitle("Create new User Certificate 1/1");
			setDescription("Use this Wizard to create a new User Certificates with one click");
		} else {
			setTitle("Create new Root Certificate 1/2");
			setDescription("Use this Wizard to create a new Root Certificate");
		}
		setPageComplete(false);

	}

	public void createControl(Composite parent) {

		CurveParams[] params = CurveRegistry.curveParams;

		Composite composite = new Composite(parent, SWT.NULL);

		GridLayout layout = new GridLayout(1, false);
		composite.setLayout(layout);

		//One Click Certificate

		if (certificateJobDescription.getType() == CertificateJobDescription.OneClick) {

			commonNameSet = false;

			Group commonNameGroup = new Group(composite, SWT.NONE);
			commonNameGroup.setText("Common Name");
			commonNameGroup.setLayout(new GridLayout(2, false));
			commonNameGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label labelCommonName = new Label(commonNameGroup, SWT.NULL);
			labelCommonName.setText("Common Name:");

			// Common Name of Certificate
			textCommonName = new Text(commonNameGroup, SWT.BORDER);
			textCommonName.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label labelKeyStrength = new Label(commonNameGroup, SWT.NULL);
			labelKeyStrength.setText("Key Strength:");

			// Key Strength
			scale = new Scale(commonNameGroup, SWT.None);
			scale.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			scale.setMaximum(2);
			scale.setPageIncrement(1);

			Label labelKSILabel = new Label(commonNameGroup, SWT.NULL);
			labelKSILabel.setText("");
			labelKeyStrengthValue = new Label(commonNameGroup, SWT.NULL);
			labelKeyStrengthValue
					.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
			labelKeyStrengthValue.setAlignment(SWT.LEFT);
			labelKeyStrengthValue.setText("Weak");

			certificateJobDescription.setKeyLength(1024);

			Label labelPW = new Label(commonNameGroup, SWT.NULL);
			labelPW.setText("PKCS#12 Password:");
			textPW = new Text(commonNameGroup, SWT.BORDER);
			textPW.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			//Listener for Common Name Field
			Listener commonNameListener = new Listener() {

				public void handleEvent(Event event) {
					String cn = textCommonName.getText();
					if (!cn.equals("")) {
						setErrorMessage(null);
						commonNameSet = true;
						certificateJobDescription.setCommonName(cn);
					} else {
						setErrorMessage("Please provide a Common Name!");
						commonNameSet = false;
					}
					checkFinished();
				}
			};

			//Listener for Password Box
			Listener passwordListener = new Listener() {

				public void handleEvent(Event event) {
					String pkcspw = textPW.getText();
					if (!pkcspw.equals("")) {
						setErrorMessage(null);
						pkcs12PasswordSet = true;

					} else {
						setErrorMessage("Please enter a password!");
						pkcs12PasswordSet = false;
					}
					checkFinished();
				}
			};

			//Listener for Key Strength

			SelectionListener keyStrengthListener = new SelectionListener() {

				public void widgetDefaultSelected(SelectionEvent e) {
					// nothing to do

				}

				public void widgetSelected(SelectionEvent e) {
					if (scale.getSelection() == 0) {
						labelKeyStrengthValue.setText("Weak");
						labelKeyStrengthValue.setAlignment(SWT.LEFT);
						certificateJobDescription.setKeyLength(1024);
					}
					if (scale.getSelection() == 1) {
						labelKeyStrengthValue.setText("Normal");
						labelKeyStrengthValue.setAlignment(SWT.CENTER);
						certificateJobDescription.setKeyLength(2048);
					}
					if (scale.getSelection() == 2) {
						labelKeyStrengthValue.setText("Strong");
						labelKeyStrengthValue.setAlignment(SWT.RIGHT);
						certificateJobDescription.setKeyLength(4096);
					}

				}

			};

			scale.addSelectionListener(keyStrengthListener);
			textCommonName.addListener(SWT.KeyUp, commonNameListener);
			textPW.addListener(SWT.KeyUp, passwordListener);

			setControl(composite);

		} else {

			if (certificateJobDescription.getType() == CertificateJobDescription.BatchMode) {

				Group groupFileName = new Group(composite, SWT.None);
				groupFileName.setLayout(new GridLayout(3, false));
				groupFileName.setText("Certificate Filename");
				if (certificateJobDescription.getType() == CertificateJobDescription.BatchMode) {
					groupFileName.setText("Batch Filename");
				}
				groupFileName.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

				Label labelFileName = new Label(groupFileName, SWT.NULL);
				labelFileName.setText("Filename:");

				Composite composite0 = new Composite(groupFileName, SWT.NULL);
				composite0.setLayout(new GridLayout(1, false));
				composite0.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

				textFileName = new Text(composite0, SWT.BORDER);
				textFileName.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
				//textFileName.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL|GridData.GRAB_HORIZONTAL));

				Button buttonFileOpen = new Button(groupFileName, SWT.NULL);
				buttonFileOpen.setText("Open");
				buttonFileOpen.addSelectionListener(new SelectionListener() {

					public void widgetDefaultSelected(SelectionEvent arg0) {

					}

					public void widgetSelected(SelectionEvent arg0) {
						/*
						 * @see http://www.java2s.com/Code/Java/SWT-JFace-Eclipse/FileDialogExample.htm
						 * and
						 * @see http://help.eclipse.org/help33/index.jsp?topic=/org.eclipse.platform.doc.isv/reference/api/org/eclipse/swt/widgets/FileDialog.html
						 */
						FileDialog fd = new FileDialog(getShell(), SWT.OPEN);
						fd.setText("Open");
						String filepath = fd.open();
						textFileName.setText(filepath);
						fileNameSet = true;
						checkFinished();
						//System.out.println("FD Open: " + textFileName.getText());
					}

				});
			}

			Group groupIssuer = new Group(composite, SWT.None);
			groupIssuer.setLayout(new GridLayout(2, false));
			groupIssuer.setText("Issuer Information");
			groupIssuer.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label labelIssuerCertificate = new Label(groupIssuer, SWT.LEFT);
			labelIssuerCertificate.setText("Issuer Certificate:");
			//labelIssuerCertificate.setLayoutData(new GridData(130,SWT.DEFAULT));

			Composite composite1 = new Composite(groupIssuer, SWT.NULL);
			composite1.setLayout(new GridLayout(1, false));
			composite1.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			comboIssuer = new Combo(composite1, SWT.READ_ONLY);
			comboIssuer.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
			//comboIssuer.add("root");
			//comboIssuer.add("ca");

			int size = KeyStoreProvider.getInstance().getCaCertificateSerials().size();
			List<BigInteger> serials = KeyStoreProvider.getInstance().getCaCertificateSerials();
			for (int i = 0; i < size; i++) {
				comboIssuer.add(serials.get(i).toString());
			}

			Label labelIssuerPassword = new Label(groupIssuer, SWT.LEFT);
			labelIssuerPassword.setText("Password:");
			//labelIssuerPassword.setLayoutData(new GridData(130,SWT.DEFAULT));

			Composite composite2 = new Composite(groupIssuer, SWT.NULL);
			composite2.setLayout(new GridLayout(1, false));
			composite2.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			textIssuerPassword = new Text(composite2, SWT.BORDER | SWT.PASSWORD);
			textIssuerPassword.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			if (certificateJobDescription.getType() == CertificateJobDescription.RootCertificate) {
				groupIssuer.setVisible(false);
			}

			Group groupKeySettings = new Group(composite, SWT.None);
			groupKeySettings.setLayout(new GridLayout(2, false));
			groupKeySettings.setText("Key Settings");
			groupKeySettings.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label labelKeyAlgorithm = new Label(groupKeySettings, SWT.LEFT);
			labelKeyAlgorithm.setText("Key Algorithm:");
			//labelKeyAlgorithm.setLayoutData(new GridData(130,SWT.DEFAULT));

			Composite composite3 = new Composite(groupKeySettings, SWT.NULL);
			composite3.setLayout(new GridLayout(1, false));
			composite3.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			comboKeyAlgorithm = new Combo(composite3, SWT.READ_ONLY);
			comboKeyAlgorithm.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
			CryptoInformationProvider cip = CryptoInformationProvider.getInstance();
			asymAlgos = cip.getValidAsymmetricAlgorithms();

			String[] algos = Application.getSettings().getArray("WorkingAlgorithms");
			for (int i = 0; i < algos.length; i++) {
				String current = algos[i];
				int cut = current.indexOf(",");
				//int cut2 = current.indexOf(",", cut + 1); // TODO remove block if not needed -ds		
				String edited = current.substring(0, cut);
				comboKeyAlgorithm.add(edited);
			}

			Label labelKeyLength = new Label(groupKeySettings, SWT.LEFT);
			labelKeyLength.setText("Key Length:");
			//labelKeyLength.setLayoutData(new GridData(130,SWT.DEFAULT));

			Composite composite4 = new Composite(groupKeySettings, SWT.NULL);
			composite4.setLayout(new GridLayout(1, false));
			composite4.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			comboKeyLength = new Combo(composite4, SWT.READ_ONLY);
			comboKeyLength.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label labelParameters = new Label(groupKeySettings, SWT.LEFT);
			labelParameters.setText("EC Curve:");

			Composite compositeEC = new Composite(groupKeySettings, SWT.NULL);
			compositeEC.setLayout(new GridLayout(1, false));
			compositeEC.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			comboCurve = new Combo(compositeEC, SWT.NONE);
			for (CurveParams param : params) {
				comboCurve.add(param.getOID().toString());
			}
			comboCurve.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label SignatureAlgorithm = new Label(groupKeySettings, SWT.LEFT);
			SignatureAlgorithm.setText("Signature Algorithm:");
			//labelKeyAlgorithm.setLayoutData(new GridData(130,SWT.DEFAULT));

			Composite composite7 = new Composite(groupKeySettings, SWT.NULL);
			composite7.setLayout(new GridLayout(1, false));
			composite7.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			comboSignatureAlgorithm = new Combo(composite7, SWT.READ_ONLY);
			comboSignatureAlgorithm.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.GRAB_HORIZONTAL));
			signAlgos = cip.getValidSignatureAlgorithms();
			SortedSet<String> signAlgosSorted = new TreeSet<String>();
			signAlgosSorted.addAll(signAlgos);

			String[] signatureAlgos = Application.getSettings().getArray("WorkingSignatures");
			for (int i = 0; i < signatureAlgos.length; i++) {
				String algo = signatureAlgos[i].substring(0, signatureAlgos[i].indexOf(","));
				comboSignatureAlgorithm.add(algo);
			}

			Group groupPKCS12 = new Group(composite, SWT.None);
			groupPKCS12.setLayout(new GridLayout(2, false));
			groupPKCS12.setText("PKCS12 Settings");
			groupPKCS12.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label labelPKCS12Password = new Label(groupPKCS12, SWT.LEFT);
			labelPKCS12Password.setText("Password:");
			//labelPKCS12Password.setLayoutData(new GridData(130,SWT.DEFAULT));

			Composite composite5 = new Composite(groupPKCS12, SWT.NULL);
			composite5.setLayout(new GridLayout(1, false));
			composite5.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			textPKCS12Password = new Text(composite5, SWT.BORDER | SWT.PASSWORD);
			textPKCS12Password.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Label labelPKCS12Password2 = new Label(groupPKCS12, SWT.LEFT);
			labelPKCS12Password2.setText("Repeat Password:");
			//labelPKCS12Password2.setLayoutData(new GridData(130,SWT.DEFAULT));

			Composite composite6 = new Composite(groupPKCS12, SWT.NULL);
			composite6.setLayout(new GridLayout(1, false));
			composite6.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			textPKCS12Password2 = new Text(composite6, SWT.BORDER | SWT.PASSWORD);
			textPKCS12Password2.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

			Listener pkcs12PasswordListener = new Listener() {

				public void handleEvent(Event event) {
					if ((textPKCS12Password.getText()).equals(textPKCS12Password2.getText())) {
						setErrorMessage(null);
						pkcs12PasswordSet = true;
					} else {
						//setPageComplete(false);
						setErrorMessage("PKCS12 Passwords do not match!");
						pkcs12PasswordSet = false;
					}
					checkFinished();
				}

			};

			Listener fileNameListener = new Listener() {

				public void handleEvent(Event event) {
					String filename = textFileName.getText();
					if (!filename.equals("")) {
						setErrorMessage(null);
						fileNameSet = true;
					} else {
						setErrorMessage("Please insert a Filename!");
						fileNameSet = false;
					}
					checkFinished();
				}
			};

			Listener issuerPasswordListener = new Listener() {

				public void handleEvent(Event event) {
					String filename = textIssuerPassword.getText();
					if (!filename.equals("")) {
						setErrorMessage(null);
						issuerPasswordSet = true;
					} else {
						setErrorMessage("Please provide the Issuer's PKCS12 Password to unlock the private Key!");
						issuerPasswordSet = false;
					}
					checkFinished();
				}
			};

			SelectionListener algorithmChangeListener = new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent arg0) {
					// NO-OP
				}

				public void widgetSelected(SelectionEvent arg0) {
					//System.out.println("algoChangeListener: " + comboKeyAlgorithm.getText());
					Set<Integer> strengthsTmp = asymAlgos.get(comboKeyAlgorithm.getText());
					SortedSet<Integer> strengths = new TreeSet<Integer>();
					strengths.addAll(strengthsTmp);
					comboKeyLength.removeAll();
					for (Iterator<Integer> iterator = strengths.iterator(); iterator.hasNext();) {
						Integer i = (Integer) iterator.next();
						comboKeyLength.add(String.valueOf(i));
					}
					CryptoInformationProvider cip = CryptoInformationProvider.getInstance();

					//System.out.println("hier");
					Set<String> signAlgos = cip.getValidSignatureAlgorithmsFor(comboKeyAlgorithm.getText());

					SortedSet<String> signAlgosSorted = new TreeSet<String>();
					signAlgosSorted.addAll(signAlgos);
					comboSignatureAlgorithm.removeAll();
					for (Iterator<String> iterator2 = signAlgosSorted.iterator(); iterator2.hasNext();) {
						String string = (String) iterator2.next();
						comboSignatureAlgorithm.add(string);
					}
					comboKeyLength.setEnabled(true);
					if (comboKeyAlgorithm.getText().contains("EC")) {
						comboCurve.setEnabled(true);
					} else {
						comboCurve.setEnabled(false);

					}
				}
			};

			SelectionListener keyAlgorithmSelectionListener = new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
					// do nothing
				}

				public void widgetSelected(SelectionEvent e) {
					Set<Integer> strengthsTmp = asymAlgos.get(comboKeyAlgorithm.getText());
					SortedSet<Integer> strengths = new TreeSet<Integer>();
					strengths.addAll(strengthsTmp);
					comboKeyLength.removeAll();
					for (Iterator<Integer> iterator = strengths.iterator(); iterator.hasNext();) {
						Integer i = (Integer) iterator.next();
						comboKeyLength.add(String.valueOf(i));
					}
					comboKeyLength.setEnabled(true);
					if (comboKeyAlgorithm.getText().contains("EC")) {
						comboCurve.setEnabled(true);
					} else {
						comboCurve.setEnabled(false);

					}
				}
			};

			SelectionListener issuerSelectionListener = new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent arg0) {
					// NO-OP
				}

				public void widgetSelected(SelectionEvent arg0) {
					setErrorMessage(null);
					pathLengthValid = true;
					codec.x509.X509Certificate issuer = KeyStoreProvider.getInstance().getCertificate(
							new BigInteger(comboIssuer.getText())).getCertificate();
					CertificateHelper helper = new CertificateHelper(issuer);
					if ((certificateJobDescription.getType() != CertificateJobDescription.ClientCertificate)
							|| (certificateJobDescription.getType() != CertificateJobDescription.BatchMode)) {

						if (issuer != null) {
							if (helper.getPathLength() == "0") {
								setErrorMessage("Path Length Limit reached!");
								pathLengthValid = false;
								checkFinished();
							} else {
								setErrorMessage(null);
								pathLengthValid = true;
								checkFinished();
							}
						}
					}

					//Fill Signature Algorithms
					CryptoInformationProvider cip = CryptoInformationProvider.getInstance();
					X509Certificate issuercert = KeyStoreProvider.getInstance().getCertificate(
							new BigInteger(comboIssuer.getText())).getCertificate();
					String issuerKeyAlgo = issuercert.getPublicKey().getAlgorithm();
					Set<String> signAlgos = cip.getValidSignatureAlgorithmsFor(issuerKeyAlgo);
					SortedSet<String> signAlgosSorted = new TreeSet<String>();
					signAlgosSorted.addAll(signAlgos);
					comboSignatureAlgorithm.removeAll();
					for (Iterator<String> iterator2 = signAlgosSorted.iterator(); iterator2.hasNext();) {
						String string = (String) iterator2.next();
						comboSignatureAlgorithm.add(string);
					}
				}
			};

			SelectionListener curveSelectionListener = new SelectionListener() {

				public void widgetDefaultSelected(SelectionEvent e) {
					// do nothing

				}

				public void widgetSelected(SelectionEvent e) {
					comboKeyLength.setEnabled(false);
					keyLengthSelected = true;
				}

			};

			SelectionListener comboSelectionListener = new SelectionListener() {

				public void widgetDefaultSelected(SelectionEvent arg0) {
					// NO-OP
				}

				public void widgetSelected(SelectionEvent arg0) {

					if ((comboKeyAlgorithm.getText()) != "") {
						algorithmSelected = true;
					}
					if ((comboKeyLength.getText()) != "") {
						keyLengthSelected = true;
						//System.out.println("setTrue");
						//for EC
						if (comboKeyAlgorithm.getText().contains("EC")) {
							int keylength = Integer.valueOf(comboKeyLength.getText());
							comboCurve.select(comboCurve.indexOf(CurveRegistry.getDefaultCurveParams(keylength)
									.getOID().toString()));
						}
					}
					if (comboIssuer.getText() != "") {
						issuerSelected = true;
						certificateJobDescription.setIssuer(comboIssuer.getText());
					}
					if ((comboSignatureAlgorithm.getText() != "")) {
						signatureAlgorithmSelected = true;
					}
					checkFinished();
				}

			};

			textPKCS12Password.addListener(SWT.KeyUp, pkcs12PasswordListener);
			textPKCS12Password2.addListener(SWT.KeyUp, pkcs12PasswordListener);
			textIssuerPassword.addListener(SWT.KeyUp, issuerPasswordListener);
			comboKeyAlgorithm.addSelectionListener(comboSelectionListener);
			comboSignatureAlgorithm.addSelectionListener(comboSelectionListener);
			comboKeyLength.addSelectionListener(comboSelectionListener);
			comboIssuer.addSelectionListener(comboSelectionListener);

			if (!(certificateJobDescription.getType() == CertificateJobDescription.RootCertificate)) {
				comboKeyAlgorithm.addSelectionListener(keyAlgorithmSelectionListener);
				comboIssuer.addSelectionListener(issuerSelectionListener);
			}

			if (certificateJobDescription.getType() == CertificateJobDescription.RootCertificate) {
				comboKeyAlgorithm.addSelectionListener(algorithmChangeListener);
			}

			if (certificateJobDescription.getType() == CertificateJobDescription.BatchMode) {
				textFileName.addListener(SWT.KeyUp, fileNameListener);

			} else {
				fileNameSet = true;
			}

			//Fill Fields Button
			/*
			SelectionListener fillListener = new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
					// do nothing
				}

				public void widgetSelected(SelectionEvent e) {
					if (certificateJobDescription.getCertificateType() == CertificateJobDescription.RootCertificate) {
						textFileName.setText("root");
					} else if (certificateJobDescription.getCertificateType() == CertificateJobDescription.CACertificate) {
						textFileName.setText("ca");
					}
					textIssuerPassword.setText("test");
					textPKCS12Password.setText("test");
					textPKCS12Password2.setText("test");
					// select RSA
					String[] itms = comboKeyAlgorithm.getItems();
					int rsaPos = 0;
					for (int i = 0; i < itms.length; i++) {
						if (itms[i].equals("RSA")) {
							rsaPos = i;
							break;
						}
					}
					comboKeyAlgorithm.select(rsaPos);
					comboIssuer.select(0);
					// select 1024
					Set<Integer> strengthsTmp = asymAlgos.get(comboKeyAlgorithm.getText());
					SortedSet<Integer> strengths = new TreeSet<Integer>();
					strengths.addAll(strengthsTmp);
					comboKeyLength.removeAll();
					for (Iterator<Integer> iterator = strengths.iterator(); iterator.hasNext();) {
						Integer i = (Integer) iterator.next();
						comboKeyLength.add(String.valueOf(i));
					}
					itms = comboKeyLength.getItems();
					int lengthPos = 0;
					for (int i = 0; i < itms.length; i++) {
						if (itms[i].equals("1024")) {
							lengthPos = i;
							break;
						}
					}
					comboKeyLength.select(lengthPos);
					// select MD5withRSA
					itms = comboSignatureAlgorithm.getItems();
					int signPos = 0;
					for (int i = 0; i < itms.length; i++) {
						if (itms[i].equals("MD5withRSA")) {
							signPos = i;
							break;
						}
					}
					comboSignatureAlgorithm.select(signPos);
					setPageComplete(true);
				}
			};
			
			Button buttonFill = new Button(composite, SWT.NONE);
			buttonFill.setText("Fill Fields");
			buttonFill.addSelectionListener(fillListener);

			Button buttonEnable = new Button(composite, SWT.NONE);
			buttonEnable.setText("Enable");
			buttonEnable.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent arg0) {
					// do nothing
				}

				public void widgetSelected(SelectionEvent arg0) {
					setPageComplete(true);
					pageComplete();
				}
			});
			*/
			
			
			comboCurve.addSelectionListener(curveSelectionListener);
			comboCurve.setEnabled(false);
			
			setControl(composite);
			
		}
		
		setPageComplete(false);
	}

	public void pageComplete() {
		if (certificateJobDescription.getType() == CertificateJobDescription.OneClick) {
			String subject[] = Application.getSettings().getArray("OneClickDefaultDN");
			subject[0] = textCommonName.getText();
			certificateJobDescription.setSubject(subject);
			certificateJobDescription.setAlgorithm(Application.getSettings().get("OneClickDefaultKeyAlgorithm"));
			certificateJobDescription.setSignatureAlgorithm(Application.getSettings().get("OneClickDefaultSignatureAlgorithm"));
			certificateJobDescription.setPKCS12Password(textPW.getText());
			certificateJobDescription.setIssuerPassword(textPW.getText());
			GregorianCalendar gcstart = new GregorianCalendar();
			GregorianCalendar gcend = new GregorianCalendar();
			gcend.add(Calendar.YEAR, 1);
			//System.out.println(gcstart);
			//System.out.println(gcend);
			certificateJobDescription.setValidNotBefore(gcstart);
			certificateJobDescription.setValidNotAfter(gcend);

		} else {

			if (certificateJobDescription.getType() == CertificateJobDescription.BatchMode) {
				certificateJobDescription.setFileName(textFileName.getText());
			}

			if (comboKeyLength.getText() == "") {
				certificateJobDescription.setCiperData(comboKeyAlgorithm.getText(), 1);
			} else {
				certificateJobDescription.setCiperData(comboKeyAlgorithm.getText(), Integer.valueOf(comboKeyLength
						.getText()));
			}
			certificateJobDescription.setECCurve(comboCurve.getText());
			certificateJobDescription.setPKCS12Password(textPKCS12Password2.getText());
			certificateJobDescription.setSignatureAlgorithm(comboSignatureAlgorithm.getText());
			certificateJobDescription.setIssuerPassword(textIssuerPassword.getText());
			certificateJobDescription.setIssuer(comboIssuer.getText());
		}
	}

	protected void checkFinished() {

		if (certificateJobDescription.getType() == CertificateJobDescription.RootCertificate) {

			issuerSelected = true;
			issuerPasswordSet = true;

		} else if (certificateJobDescription.getType() == CertificateJobDescription.OneClick) {

			keyLengthSelected = true;
			issuerSelected = true;
			pathLengthValid = true;
			fileNameSet = true;
			algorithmSelected = true;
			issuerPasswordSet = true;
			signatureAlgorithmSelected = true;

		}

		if (algorithmSelected && fileNameSet && issuerPasswordSet && issuerSelected && keyLengthSelected
				&& pkcs12PasswordSet && signatureAlgorithmSelected && pathLengthValid && commonNameSet) {

			if ((certificateJobDescription.getType() == CertificateJobDescription.BatchMode)
					|| (certificateJobDescription.getType() == CertificateJobDescription.OneClick)) {
				certificateJobDescription.setComplete();
			}

			setPageComplete(true);

		} else {
			certificateJobDescription.setInComplete();
			setPageComplete(false);
		}

	}

}
