package minica.wizards;

import minica.cert.CertificateJobDescription;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This class encapsulates all the information needed to create one or more Certificates.
 * Use it for CertificateJob.
 * @author stephan
 */

public class CertificateWizardKeyUsagePage extends WizardPage {

	private CertificateJobDescription	certificateJobDescription;

	private Button						buttonDigitalSignatureYes;

	private Button						buttonOnlyDecryptionYes;

	private Button						buttonOnlyEncryptionYes;

	private Button						buttonNonRepudabilityYes;

	private Button						buttonKeyAgreementYes;

	private Button						buttonDataEncryptionYes;

	private Button						buttonKeyEncryptionYes;

	protected CertificateWizardKeyUsagePage(String pageName) {
		super(pageName);
	}

	protected CertificateWizardKeyUsagePage(String pageName, CertificateJobDescription data) {

		super(pageName);
		certificateJobDescription = data;

		int type = certificateJobDescription.getType();

		if (type == CertificateJobDescription.CACertificate) {
			setTitle("Create new CA Certificate 2/2");
			setDescription("Use this Wizard to create a new CA Certificate");
		} else if (type == CertificateJobDescription.ClientCertificate) {
			setTitle("Create new User Certificate 3/3");
			setDescription("Use this Wizard to create a new User Certificate");
		} else {
			setTitle("Create new Root Certificate 2/2");
			setDescription("Use this Wizard to create a new Root Certificate");
		}

	}

	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.NULL);

		GridLayout layout = new GridLayout(1, false);
		composite.setLayout(layout);

		Group groupBasicConstraints = new Group(composite, SWT.NULL);
		groupBasicConstraints.setLayout(new GridLayout(2, false));
		groupBasicConstraints.setText("Basic Constraints");
		groupBasicConstraints.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelDigitalSignature = new Label(groupBasicConstraints, SWT.NULL);
		labelDigitalSignature.setText("Digital Signature");

		Composite composite1 = new Composite(groupBasicConstraints, SWT.NULL);
		composite1.setLayout(new GridLayout(2, false));

		buttonDigitalSignatureYes = new Button(composite1, SWT.RADIO);
		buttonDigitalSignatureYes.setText("Yes");
		Button buttonDigitalSignatureNo = new Button(composite1, SWT.RADIO);
		buttonDigitalSignatureNo.setText("No");

		Label labelKeyEncryption = new Label(groupBasicConstraints, SWT.NULL);
		labelKeyEncryption.setText("Key Encryption");

		Composite composite2 = new Composite(groupBasicConstraints, SWT.NULL);
		composite2.setLayout(new GridLayout(3, false));

		buttonKeyEncryptionYes = new Button(composite2, SWT.RADIO);
		buttonKeyEncryptionYes.setText("Yes");
		Button buttonKeyEncryptionNo = new Button(composite2, SWT.RADIO);
		buttonKeyEncryptionNo.setText("No");

		Label labelDataEncryption = new Label(groupBasicConstraints, SWT.NULL);
		labelDataEncryption.setText("Data Encryption");

		Composite composite3 = new Composite(groupBasicConstraints, SWT.NULL);
		composite3.setLayout(new GridLayout(3, false));

		buttonDataEncryptionYes = new Button(composite3, SWT.RADIO);
		buttonDataEncryptionYes.setText("Yes");
		Button buttonDataEncryptionNo = new Button(composite3, SWT.RADIO);
		buttonDataEncryptionNo.setText("No");

		Label labelKeyAgreement = new Label(groupBasicConstraints, SWT.NULL);
		labelKeyAgreement.setText("Key Agreement");

		Composite composite4 = new Composite(groupBasicConstraints, SWT.NULL);
		composite4.setLayout(new GridLayout(3, false));

		buttonKeyAgreementYes = new Button(composite4, SWT.RADIO);
		buttonKeyAgreementYes.setText("Yes");
		Button buttonKeyAgreementNo = new Button(composite4, SWT.RADIO);
		buttonKeyAgreementNo.setText("No");

		Label labelNonRepudability = new Label(groupBasicConstraints, SWT.NULL);
		labelNonRepudability.setText("Non-Repudability");

		Composite composite5 = new Composite(groupBasicConstraints, SWT.NULL);
		composite5.setLayout(new GridLayout(3, false));

		buttonNonRepudabilityYes = new Button(composite5, SWT.RADIO);
		buttonNonRepudabilityYes.setText("Yes");
		Button buttonNonRepudabilityNo = new Button(composite5, SWT.RADIO);
		buttonNonRepudabilityNo.setText("No");

		Label labelOnlyEncryption = new Label(groupBasicConstraints, SWT.NULL);
		labelOnlyEncryption.setText("Only Encryption");

		Composite composite6 = new Composite(groupBasicConstraints, SWT.NULL);
		composite6.setLayout(new GridLayout(3, false));

		buttonOnlyEncryptionYes = new Button(composite6, SWT.RADIO);
		buttonOnlyEncryptionYes.setText("Yes");
		Button buttonOnlyEncryptionNo = new Button(composite6, SWT.RADIO);
		buttonOnlyEncryptionNo.setText("No");

		Label labelOnlyDecryption = new Label(groupBasicConstraints, SWT.NULL);
		labelOnlyDecryption.setText("Only Decryption");

		Composite composite7 = new Composite(groupBasicConstraints, SWT.NULL);
		composite7.setLayout(new GridLayout(3, false));

		buttonOnlyDecryptionYes = new Button(composite7, SWT.RADIO);
		buttonOnlyDecryptionYes.setText("Yes");
		Button buttonOnlyDecryptionNo = new Button(composite7, SWT.RADIO);
		buttonOnlyDecryptionNo.setText("No");

		buttonDataEncryptionYes.setSelection(true);
		buttonDigitalSignatureYes.setSelection(true);
		buttonKeyAgreementYes.setSelection(true);
		buttonKeyEncryptionYes.setSelection(true);
		buttonNonRepudabilityYes.setSelection(true);
		buttonOnlyDecryptionNo.setSelection(true);
		buttonOnlyEncryptionNo.setSelection(true);

		setControl(composite);

	}

	public void pageComplete() {
		boolean[] b = new boolean[9];
		b[0] = buttonDigitalSignatureYes.getSelection();
		b[1] = buttonNonRepudabilityYes.getSelection();
		b[2] = buttonKeyEncryptionYes.getSelection();
		b[3] = buttonDataEncryptionYes.getSelection();
		b[4] = buttonKeyAgreementYes.getSelection();
		b[5] = false;
		b[6] = false;
		b[7] = buttonOnlyEncryptionYes.getSelection();
		b[8] = buttonOnlyDecryptionYes.getSelection();
		certificateJobDescription.setKeyUsage(b);
	}

	@Override
	public void setVisible(boolean visible) {
		certificateJobDescription.setComplete();
		super.setVisible(visible);
	}
}
