package minica.wizards;

import java.math.BigInteger;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

import minica.cert.CertificateJobDescription;
import minica.crypto.KeyStoreProvider;
import minica.util.CertificateHelper;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import codec.x509.X509Certificate;

/*
 * Copyright (c) 2007-2008 Bastian Neuburger Copyright (c) 2007-2008 Dominik Schulz Copyright (c) 2007-2008 Stephan
 * Heuser Copyright (c) 2007-2008 Sören Mess
 * 
 * This file is part of FlexiCA.
 * 
 * FlexiCA is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * FlexiCA is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with FlexiCA; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA </pre>
 */
/**
 * This is the Front Page of the Certificate Wizard
 * @author stephan
 *
 */
public class CertificateWizardPropertiesPage extends WizardPage {

	private CertificateJobDescription	certificateJobDescription;

	private Text						textCommonName;

	private Text						textOrganisationalUnit;

	private Text						textOrganisation;

	private Combo						comboCountry;

	private Combo						comboDayStart;

	private Combo						comboYearStart;

	private Combo						comboMonthStart;

	private Combo						comboYearEnd;

	private Combo						comboMonthEnd;

	private Combo						comboDayEnd;

	private Spinner						spinnerPathLength;

	private GregorianCalendar			calendarStart;

	private GregorianCalendar			calendarEnd;

	private boolean						commonNameSet		= false;

	private boolean						organisationUnitSet	= false;

	private boolean						organisationSet		= false;

	private boolean						countrySelected		= false;

	private boolean						datesValid			= true;

	private boolean						pathLengthValid		= true;

	private final String[]				Months				= new String[] { "January", "February", "March", "April",
			"May", "June", "July", "August", "September", "October", "November", "December" };

	private Button						buttonCRL;

	private Button						buttonNoCRL;

	private Text						textCRLDistributionPoint;

	protected CertificateWizardPropertiesPage(String pageName, CertificateJobDescription data) {

		super(pageName);
		certificateJobDescription = data;

		int type = certificateJobDescription.getType();

		if (type == CertificateJobDescription.CACertificate) {
			setTitle("Create new CA Certificate 2/2");
			setDescription("Use this Wizard to create a new CA Certificate");
		} else if (type == CertificateJobDescription.ClientCertificate) {
			setTitle("Create new User Certificate 2/3");
			setDescription("Use this Wizard to create a new User Certificate");
		} else {
			setTitle("Create new Root Certificate 2/2");
			setDescription("Use this Wizard to create a new Root Certificate");
		}

	}

	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.NULL);

		GridLayout layout = new GridLayout(1, false);
		composite.setLayout(layout);

		Group groupSubjectInformation = new Group(composite, SWT.NULL);
		groupSubjectInformation.setLayout(new GridLayout(2, false));
		groupSubjectInformation.setText("Subject Information");
		groupSubjectInformation.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelCommonName = new Label(groupSubjectInformation, SWT.NULL);
		labelCommonName.setText("Common Name:");
		textCommonName = new Text(groupSubjectInformation, SWT.BORDER);
		textCommonName.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelOrganisationalUnit = new Label(groupSubjectInformation, SWT.NULL);
		labelOrganisationalUnit.setText("Organisational Unit:");
		textOrganisationalUnit = new Text(groupSubjectInformation, SWT.BORDER);
		textOrganisationalUnit.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelOrganisation = new Label(groupSubjectInformation, SWT.NULL);
		labelOrganisation.setText("Organisation:");
		textOrganisation = new Text(groupSubjectInformation, SWT.BORDER);
		textOrganisation.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelCountry = new Label(groupSubjectInformation, SWT.NULL);
		labelCountry.setText("Country:");
		comboCountry = new Combo(groupSubjectInformation, SWT.READ_ONLY);
		comboCountry.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
		
		// This retrieves a list of all ISO3166 country abbreviations available from 
		//java.util.Locale and adds them to the comboCountry combo box
		
		String isoCountries[] = Locale.getISOCountries();
		String defaultCountry = Locale.getDefault().getCountry();
		int defaultCountryIndex = 55;
		for (int i = 0; i < isoCountries.length; i++) {
			comboCountry.add(isoCountries[i]);
			if (isoCountries[i].equals(defaultCountry)) {
				defaultCountryIndex = i;
			}
		}
		countrySelected = true;
		comboCountry.select(defaultCountryIndex);

		Group groupValidityPeriod = new Group(composite, SWT.NULL);
		groupValidityPeriod.setLayout(new GridLayout(6, false));
		groupValidityPeriod.setText("Validity Period");
		groupValidityPeriod.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelDayStart = new Label(groupValidityPeriod, SWT.NULL);
		labelDayStart.setText("Not Before: Day:");
		comboDayStart = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboDayStart.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelMonthStart = new Label(groupValidityPeriod, SWT.NULL);
		labelMonthStart.setText("Month:");
		comboMonthStart = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboMonthStart.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelYearStart = new Label(groupValidityPeriod, SWT.NULL);
		labelYearStart.setText("Year:");
		comboYearStart = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboYearStart.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelDayEnd = new Label(groupValidityPeriod, SWT.NULL);
		labelDayEnd.setText("Not After: Day:");
		comboDayEnd = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboDayEnd.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelMonthEnd = new Label(groupValidityPeriod, SWT.NULL);
		labelMonthEnd.setText("Month:");
		comboMonthEnd = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboMonthEnd.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelYearEnd = new Label(groupValidityPeriod, SWT.NULL);
		labelYearEnd.setText("Year:");
		comboYearEnd = new Combo(groupValidityPeriod, SWT.READ_ONLY);
		comboYearEnd.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Group groupCA = new Group(composite, SWT.NULL);
		groupCA.setLayout(new GridLayout(2, false));
		groupCA.setText("CA Information");
		groupCA.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		Label labelCA = new Label(groupCA, SWT.NULL);
		labelCA.setText("Path Length:");
		//labelCA.setLayoutData(new GridData(130,SWT.DEFAULT));

		spinnerPathLength = new Spinner(groupCA, SWT.BORDER);
		spinnerPathLength.setLayoutData(new GridData());
		spinnerPathLength.setMinimum(0);
		spinnerPathLength.setMaximum(100);
		spinnerPathLength.setSelection(1);

		Label labelCRL = new Label(groupCA, SWT.NULL);
		labelCRL.setText("Use for CRL Signing:");
		//labelCRL.setLayoutData(new GridData(130, SWT.DEFAULT));

		buttonCRL = new Button(groupCA, SWT.RADIO);
		@SuppressWarnings("unused")
		Label placeholder = new Label(groupCA, SWT.NULL);
		buttonNoCRL = new Button(groupCA, SWT.RADIO);
		buttonCRL.setText("Yes");
		buttonNoCRL.setText("No");

		Label LabelCRLDistributionPoint = new Label(groupCA, SWT.NULL);
		LabelCRLDistributionPoint.setText("CRL Distribution Point:");
		textCRLDistributionPoint = new Text(groupCA, SWT.BORDER);
		textCRLDistributionPoint.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		if (certificateJobDescription.getType() == CertificateJobDescription.ClientCertificate) {
			groupCA.setVisible(false);
		}

		SelectionListener ComboSelectionListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			public void widgetSelected(SelectionEvent arg0) {

				if ((comboCountry.getText()) != null) {
					countrySelected = true;
				}
				checkFinished();
			}

		};

		Listener organisationListener = new Listener() {

			public void handleEvent(Event event) {
				String filename = textOrganisation.getText();
				if (!filename.equals("")) {
					setErrorMessage(null);
					organisationSet = true;
				} else {
					setErrorMessage("Please provide an Organisation!");
					organisationSet = false;
				}
				checkFinished();
			}
		};

		Listener organisationalUnitListener = new Listener() {

			public void handleEvent(Event event) {
				String filename = textOrganisationalUnit.getText();
				if (!filename.equals("")) {
					setErrorMessage(null);
					organisationUnitSet = true;
				} else {
					setErrorMessage("Please provide an Organisational Unit!");
					organisationUnitSet = false;
				}
				checkFinished();
			}
		};

		Listener commonNameListener = new Listener() {

			public void handleEvent(Event event) {
				String filename = textCommonName.getText();
				if (!filename.equals("")) {
					setErrorMessage(null);
					commonNameSet = true;
				} else {
					setErrorMessage("Please provide a Common Name!");
					commonNameSet = false;
				}
				checkFinished();
			}
		};

		SelectionListener DateCheckListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				int month_start = comboMonthStart.getSelectionIndex();
				int year_start = new Integer(comboYearStart.getText()).intValue();
				int day_start = comboDayStart.getSelectionIndex() + 1;

				int month_end = comboMonthStart.getSelectionIndex();
				int year_end = new Integer(comboYearEnd.getText()).intValue();
				int day_end = comboDayEnd.getSelectionIndex() + 1;

				GregorianCalendar tempcal = new GregorianCalendar();
				tempcal.set(Calendar.DATE, 1);
				tempcal.set(Calendar.MONTH, month_start);
				tempcal.set(Calendar.YEAR, year_start);

				int max_day = tempcal.getActualMaximum(Calendar.DATE);

				comboDayStart.removeAll();

				for (int i = 1; i < max_day + 1; i++) {
					comboDayStart.add(((Integer) i).toString());
				}

				if (day_start > max_day) {
					day_start = max_day;
					comboDayStart.select(max_day - 1);
				} else {
					comboDayStart.select(day_start - 1);
				}

				calendarStart.set(year_start, month_start, day_start);

				tempcal.set(Calendar.DATE, 1);
				tempcal.set(Calendar.MONTH, month_end);
				tempcal.set(Calendar.YEAR, year_end);

				max_day = tempcal.getActualMaximum(Calendar.DATE);

				for (int i = 1; i < max_day + 1; i++) {
					comboDayEnd.add(((Integer) i).toString());
				}

				if (day_end > max_day) {
					day_end = max_day;
					comboDayEnd.select(max_day - 1);
				} else {
					comboDayEnd.select(day_end - 1);
				}

				calendarEnd.set(year_end, month_end, day_end);

				if (!(calendarStart.before(calendarEnd))) {
					setErrorMessage("Validity Period Invalid!");
					datesValid = false;
				} else {
					setErrorMessage(null);
					datesValid = true;
				}
				checkFinished();

			}

		};

		SelectionListener CRLListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				if (buttonCRL.getSelection()) {
					textCRLDistributionPoint.setEnabled(true);
				} else {
					textCRLDistributionPoint.setEnabled(false);
				}

			}

		};

		SelectionListener pathLengthListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				setErrorMessage(null);
				pathLengthValid = true;

				X509Certificate issuer = null;
				CertificateHelper helper = null;
				if (certificateJobDescription.getType() != CertificateJobDescription.RootCertificate) {
					issuer = KeyStoreProvider.getInstance().getCertificate(
							new BigInteger(certificateJobDescription.getIssuer())).getCertificate();
					helper = new CertificateHelper(issuer);
				}
				if (certificateJobDescription.getType() == CertificateJobDescription.CACertificate) {
					if (spinnerPathLength.getSelection() >= new Integer(helper.getPathLength()).intValue()) {
						setErrorMessage("Path Length Limit reached!");
						pathLengthValid = false;
						checkFinished();
					} else {
						setErrorMessage(null);
						pathLengthValid = true;
						checkFinished();
					}
				}

			}

		};
		/*
		SelectionListener fillListener = new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing

			}

			public void widgetSelected(SelectionEvent e) {
				if (certificateJobDescription.getCertificateType() == CertificateJobDescription.RootCertificate) {
					textCommonName.setText("CNRoot");
					textCRLDistributionPoint.setText("http://www.crl.de/crl");
					textOrganisation.setText("ORoot");
					textOrganisationalUnit.setText("OURoot");
				} else if (certificateJobDescription.getCertificateType() == CertificateJobDescription.CACertificate) {
					textCommonName.setText("CNCA");
					textCRLDistributionPoint.setText("http://www.crl.de/crl");
					textOrganisation.setText("OCA");
					textOrganisationalUnit.setText("OUCA");
				}

				comboCountry.select(0);

				if (!(certificateJobDescription.getType() == CertificateJobDescription.ClientCertificate)) {
					certificateJobDescription.setComplete();
				}
				setPageComplete(true);
			}

		};
		Button buttonFill = new Button(composite, SWT.NONE);
		buttonFill.setText("Fill Fields");

		buttonFill.addSelectionListener(fillListener);
		*/

		buttonCRL.addSelectionListener(CRLListener);
		buttonNoCRL.addSelectionListener(CRLListener);

		comboDayStart.addSelectionListener(DateCheckListener);
		comboMonthStart.addSelectionListener(DateCheckListener);
		comboYearStart.addSelectionListener(DateCheckListener);

		comboDayEnd.addSelectionListener(DateCheckListener);
		comboMonthEnd.addSelectionListener(DateCheckListener);
		comboYearEnd.addSelectionListener(DateCheckListener);

		comboCountry.addSelectionListener(ComboSelectionListener);
		textOrganisation.addListener(SWT.KeyUp, organisationListener);
		textOrganisationalUnit.addListener(SWT.KeyUp, organisationalUnitListener);
		textCommonName.addListener(SWT.KeyUp, commonNameListener);

		spinnerPathLength.addSelectionListener(pathLengthListener);

		initCalendar();

		setControl(composite);
		buttonNoCRL.setSelection(true);
		textCRLDistributionPoint.setEnabled(false);
		setPageComplete(false);
	}

	public void initCalendar() {

		calendarStart = new GregorianCalendar();
		calendarEnd = new GregorianCalendar();

		int currentYear = calendarStart.get(GregorianCalendar.YEAR);
		int currentMonth = calendarStart.get(GregorianCalendar.MONTH);
		int currentDay = calendarStart.get(GregorianCalendar.DATE);
		calendarEnd.set(GregorianCalendar.YEAR, currentYear + 1);
		int nextYearMonth = calendarEnd.get(GregorianCalendar.MONTH);

		comboMonthStart.setItems(Months);
		comboMonthEnd.setItems(Months);
		comboMonthStart.select(currentMonth);
		comboMonthEnd.select(nextYearMonth);

		for (int i = currentYear - 20; i < currentYear + 20; i++) {
			comboYearStart.add(((Integer) i).toString());
			comboYearEnd.add(((Integer) i).toString());
		}
		comboYearStart.select(20);
		comboYearEnd.select(21);

		int maxDaysStart = calendarStart.getActualMaximum(GregorianCalendar.DATE);
		int maxDaysEnd = calendarEnd.getActualMaximum(GregorianCalendar.DATE);

		for (int i = 1; i < maxDaysStart + 1; i++) {
			comboDayStart.add(((Integer) i).toString());
		}

		for (int i = 1; i < maxDaysEnd + 1; i++) {
			comboDayEnd.add(((Integer) i).toString());
		}
		comboDayStart.select(currentDay - 1);

		comboDayEnd.select(calendarEnd.get(GregorianCalendar.DATE) - 1);

	}

	public void pageComplete() {
		String subject[] = { textCommonName.getText(), textOrganisationalUnit.getText(), textOrganisation.getText(),
				comboCountry.getText() };
		certificateJobDescription.setSubject(subject);
		GregorianCalendar calendar[] = { calendarStart, calendarEnd };
		certificateJobDescription.setValidityPeriod(calendar);
		certificateJobDescription.setCRL(buttonCRL.getSelection(), spinnerPathLength.getSelection(),
				textCRLDistributionPoint.getText());

		if (certificateJobDescription.getCertificateType() == CertificateJobDescription.RootCertificate
				|| certificateJobDescription.getCertificateType() == CertificateJobDescription.CACertificate) {
			boolean[] b = new boolean[9];
			b[0] = false;
			b[1] = false;
			b[2] = false;
			b[3] = false;
			b[4] = false;
			b[5] = true;
			b[6] = buttonCRL.getSelection();
			b[7] = false;
			b[8] = false;
			certificateJobDescription.setKeyUsage(b);
		}
	}

	private void checkFinished() {

		if (commonNameSet && organisationSet && organisationUnitSet && countrySelected && datesValid && pathLengthValid) {
			if (!(certificateJobDescription.getType() == CertificateJobDescription.ClientCertificate)) {
				certificateJobDescription.setComplete();
			}
			setPageComplete(true);
		} else {
			if (!(certificateJobDescription.getType() == CertificateJobDescription.ClientCertificate)) {
				certificateJobDescription.setInComplete();
			}
			setPageComplete(false);
		}

	}

}
